@@ -1,4 +1,3 @@
-local H = wesnoth.require "helper"
 local AH = wesnoth.require "ai/lua/ai_helper.lua"
 local FAU = wesnoth.require "ai/micro_ais/cas/ca_fast_attack_utils.lua"
 local LS = wesnoth.require "location_set"
@@ -74,14 +73,14 @@ function ca_fast_combat_leader:evaluation(cfg, data)
     -- Enemy power and number maps
     -- Currently, the power is simply the summed hitpoints of all enemies that
     -- can get to a hex
-    local enemies = AH.get_live_units {
+    local live_enemies = AH.get_live_units {
         { "filter_side", { { "enemy_of", { side = wesnoth.current.side } } } }
     }
 
     local enemy_power_map = LS.create()
     local enemy_number_map = LS.create()
 
-    for _,enemy in ipairs(enemies) do
+    for _,enemy in ipairs(live_enemies) do
         -- Only need to consider enemies that are close enough
         if (wesnoth.map.distance_between(leader.x, leader.y, enemy.x, enemy.y) <= (enemy.max_moves + leader.max_moves + 1)) then
             local enemy_power = enemy.hitpoints
@@ -111,7 +110,7 @@ function ca_fast_combat_leader:evaluation(cfg, data)
     -- not units) on the AI leader
     local leader_current_threat = 0
     if cfg and cfg.threatened_leader_fights then
-        for xa,ya in H.adjacent_tiles(leader.x, leader.y) do
+        for xa,ya in wesnoth.current.map:iter_adjacent(leader) do
             local enemy_power = enemy_power_map:get(xa, ya) or 0
             if (enemy_power > leader_current_threat) then
                 leader_current_threat = enemy_power
@@ -122,7 +121,7 @@ function ca_fast_combat_leader:evaluation(cfg, data)
     local attacks = AH.get_attacks({ leader }, { include_occupied = cfg.include_occupied_attack_hexes, viewing_side = viewing_side, ignore_visibility = ignore_visibility })
 
     if (#attacks > 0) then
-        local max_rating, best_target, best_dst = - math.huge
+        local max_rating, best_target, best_dst = - math.huge, nil, nil
         for _,attack in ipairs(attacks) do
             if enemy_map:get(attack.target.x, attack.target.y)
                 and (not avoid_map:get(attack.dst.x, attack.dst.y))
@@ -130,7 +129,7 @@ function ca_fast_combat_leader:evaluation(cfg, data)
                 -- First check if the threat against the leader at this hex
                 -- is acceptable
                 local acceptable_attack = true
-                for xa,ya in H.adjacent_tiles(attack.dst.x, attack.dst.y) do
+                for xa,ya in wesnoth.current.map:iter_adjacent(attack.dst) do
                     local enemy_power = enemy_power_map:get(xa, ya) or 0
                     local enemy_number = enemy_number_map:get(xa, ya) or 0
 
