@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -20,7 +20,6 @@
 #include "widgets/widget.hpp"
 
 struct SDL_Rect;
-class CVideo;
 
 namespace editor {
 
@@ -39,7 +38,7 @@ struct item_group
 	std::string id;
 	t_string name;
 	std::string icon;
-    bool core;
+	bool core;
 };
 
 
@@ -47,7 +46,7 @@ class common_palette  : public gui::widget {
 
 public:
 
-	common_palette(CVideo& video) : gui::widget(video, true) {}
+	common_palette() : gui::widget(true) {}
 
 	virtual ~common_palette() {}
 
@@ -66,7 +65,6 @@ public:
 
 	//drawing
 	virtual void adjust_size(const SDL_Rect& target) = 0;
-	virtual void draw() = 0;
 
 	//group
 	virtual void set_group(std::size_t index) = 0;
@@ -77,7 +75,7 @@ public:
 	/** Menu expanding for palette group list */
 	virtual void expand_palette_groups_menu(std::vector<config>& items, int i) = 0;
 
-    //item
+	//item
 	virtual std::size_t num_items() = 0;
 	virtual std::size_t start_num() = 0;
 	virtual void set_start_item(std::size_t index) = 0;
@@ -92,8 +90,8 @@ public:
 class tristate_palette : public common_palette
 {
 public:
-	tristate_palette(CVideo& video)
-		: common_palette(video)
+	tristate_palette()
+		: common_palette()
 	{
 	}
 	virtual void select_fg_item(const std::string& item_id) = 0;
