@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -23,23 +23,21 @@
 
 #include "config.hpp"
 #include "deprecation.hpp"
-#include "game_config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include "serialization/base64.hpp"
 #include "serialization/string_utils.hpp"
 #include "serialization/unicode.hpp"
-#include "serialization/unicode_cast.hpp"
+#include "utils/general.hpp"
 
-#include <boost/algorithm/string.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/iostreams/device/file_descriptor.hpp>
 #include <boost/iostreams/stream.hpp>
+#include <boost/process.hpp>
 #include "game_config_view.hpp"
 
 #ifdef _WIN32
-#include "log_windows.hpp"
-
 #include <boost/locale.hpp>
 
 #include <windows.h>
@@ -71,9 +69,33 @@ static lg::log_domain log_filesystem("filesystem");
 #define WRN_FS LOG_STREAM(warn, log_filesystem)
 #define ERR_FS LOG_STREAM(err, log_filesystem)
 
+namespace bp = boost::process;
 namespace bfs = boost::filesystem;
 using boost::system::error_code;
 
+namespace game_config
+{
+//
+// Path info
+//
+#ifdef WESNOTH_PATH
+std::string path = WESNOTH_PATH;
+#else
+std::string path = "";
+#endif
+
+#ifdef DEFAULT_PREFS_PATH
+std::string default_preferences_path = DEFAULT_PREFS_PATH;
+#else
+std::string default_preferences_path = "";
+#endif
+bool check_migration = false;
+
+const std::string observer_team_name = "observer";
+
+int cache_compression_level = 6;
+}
+
 namespace
 {
 // These are the filenames that get special processing
@@ -169,7 +191,7 @@ public:
 		try {
 			customcodecvt_do_conversion<char, wchar_t>(state, from, from_end, from_next, to, to_end, to_next);
 		} catch(...) {
-			ERR_FS << "Invalid UTF-8 string'" << std::string(from, from_end) << "' " << std::endl;
+			ERR_FS << "Invalid UTF-8 string'" << std::string(from, from_end) << "' with exception: " << utils::get_unknown_exception_type();
 			return std::codecvt_base::error;
 		}
 
@@ -187,7 +209,7 @@ public:
 		try {
 			customcodecvt_do_conversion<wchar_t, char>(state, from, from_end, from_next, to, to_end, to_next);
 		} catch(...) {
-			ERR_FS << "Invalid UTF-16 string" << std::endl;
+			ERR_FS << "Invalid UTF-16 string with exception: " << utils::get_unknown_exception_type();
 			return std::codecvt_base::error;
 		}
 
@@ -233,6 +255,40 @@ bool is_filename_case_correct(const std::string& /*fname*/, const boost::iostrea
 namespace filesystem
 {
 
+const blacklist_pattern_list default_blacklist{
+	{
+		/* Blacklist dot-files/dirs, which are hidden files in UNIX platforms */
+		".+",
+		"#*#",
+		"*~",
+		"*-bak",
+		"*.swp",
+		"*.pbl",
+		"*.ign",
+		"_info.cfg",
+		"*.exe",
+		"*.bat",
+		"*.cmd",
+		"*.com",
+		"*.scr",
+		"*.sh",
+		"*.js",
+		"*.vbs",
+		"*.o",
+		"*.ini",
+		/* Remove junk created by certain file manager ;) */
+		"Thumbs.db",
+		/* Eclipse plugin */
+		"*.wesnoth",
+		"*.project",
+	},
+	{
+		".+",
+		/* macOS metadata-like cruft (http://floatingsun.net/2007/02/07/whats-with-__macosx-in-zip-files/) */
+		"__MACOSX",
+	}
+};
+
 static void push_if_exists(std::vector<std::string>* vec, const bfs::path& file, bool full)
 {
 	if(vec != nullptr) {
@@ -254,7 +310,7 @@ static bool is_directory_internal(const bfs::path& fpath)
 	error_code ec;
 	bool is_dir = bfs::is_directory(fpath, ec);
 	if(error_except_not_found(ec)) {
-		LOG_FS << "Failed to check if " << fpath.string() << " is a directory: " << ec.message() << '\n';
+		LOG_FS << "Failed to check if " << fpath.string() << " is a directory: " << ec.message();
 	}
 
 	return is_dir;
@@ -265,7 +321,7 @@ static bool file_exists(const bfs::path& fpath)
 	error_code ec;
 	bool exists = bfs::exists(fpath, ec);
 	if(error_except_not_found(ec)) {
-		ERR_FS << "Failed to check existence of file " << fpath.string() << ": " << ec.message() << '\n';
+		ERR_FS << "Failed to check existence of file " << fpath.string() << ": " << ec.message();
 	}
 
 	return exists;
@@ -279,7 +335,7 @@ static bfs::path get_dir(const bfs::path& dirpath)
 		bfs::create_directory(dirpath, ec);
 
 		if(ec) {
-			ERR_FS << "Failed to create directory " << dirpath.string() << ": " << ec.message() << '\n';
+			ERR_FS << "Failed to create directory " << dirpath.string() << ": " << ec.message();
 		}
 
 		// This is probably redundant
@@ -287,7 +343,7 @@ static bfs::path get_dir(const bfs::path& dirpath)
 	}
 
 	if(!is_dir) {
-		ERR_FS << "Could not open or create directory " << dirpath.string() << '\n';
+		ERR_FS << "Could not open or create directory " << dirpath.string();
 		return std::string();
 	}
 
@@ -300,19 +356,19 @@ static bool create_directory_if_missing(const bfs::path& dirpath)
 	bfs::file_status fs = bfs::status(dirpath, ec);
 
 	if(error_except_not_found(ec)) {
-		ERR_FS << "Failed to retrieve file status for " << dirpath.string() << ": " << ec.message() << '\n';
+		ERR_FS << "Failed to retrieve file status for " << dirpath.string() << ": " << ec.message();
 		return false;
 	} else if(bfs::is_directory(fs)) {
-		DBG_FS << "directory " << dirpath.string() << " exists, not creating\n";
+		DBG_FS << "directory " << dirpath.string() << " exists, not creating";
 		return true;
 	} else if(bfs::exists(fs)) {
-		ERR_FS << "cannot create directory " << dirpath.string() << "; file exists\n";
+		ERR_FS << "cannot create directory " << dirpath.string() << "; file exists";
 		return false;
 	}
 
 	bool created = bfs::create_directory(dirpath, ec);
 	if(ec) {
-		ERR_FS << "Failed to create directory " << dirpath.string() << ": " << ec.message() << '\n';
+		ERR_FS << "Failed to create directory " << dirpath.string() << ": " << ec.message();
 	}
 
 	return created;
@@ -320,7 +376,7 @@ static bool create_directory_if_missing(const bfs::path& dirpath)
 
 static bool create_directory_if_missing_recursive(const bfs::path& dirpath)
 {
-	DBG_FS << "creating recursive directory: " << dirpath.string() << '\n';
+	DBG_FS << "creating recursive directory: " << dirpath.string();
 
 	if(dirpath.empty()) {
 		return false;
@@ -330,7 +386,7 @@ static bool create_directory_if_missing_recursive(const bfs::path& dirpath)
 	bfs::file_status fs = bfs::status(dirpath);
 
 	if(error_except_not_found(ec)) {
-		ERR_FS << "Failed to retrieve file status for " << dirpath.string() << ": " << ec.message() << '\n';
+		ERR_FS << "Failed to retrieve file status for " << dirpath.string() << ": " << ec.message();
 		return false;
 	} else if(bfs::is_directory(fs)) {
 		return true;
@@ -341,7 +397,7 @@ static bool create_directory_if_missing_recursive(const bfs::path& dirpath)
 	if(!dirpath.has_parent_path() || create_directory_if_missing_recursive(dirpath.parent_path())) {
 		return create_directory_if_missing(dirpath);
 	} else {
-		ERR_FS << "Could not create parents to " << dirpath.string() << '\n';
+		ERR_FS << "Could not create parents to " << dirpath.string();
 		return false;
 	}
 }
@@ -367,11 +423,11 @@ void get_files_in_dir(const std::string& dir,
 	const bfs::path dirpath(dir);
 
 	if(reorder == reorder_mode::DO_REORDER) {
-		LOG_FS << "searching for _main.cfg in directory " << dir << '\n';
+		LOG_FS << "searching for _main.cfg in directory " << dir;
 		const bfs::path maincfg = dirpath / maincfg_filename;
 
 		if(file_exists(maincfg)) {
-			LOG_FS << "_main.cfg found : " << maincfg << '\n';
+			LOG_FS << "_main.cfg found : " << maincfg;
 			push_if_exists(files, maincfg, mode == name_mode::ENTIRE_FILE_PATH);
 			return;
 		}
@@ -387,9 +443,10 @@ void get_files_in_dir(const std::string& dir,
 	}
 
 	for(; di != end; ++di) {
+		ec.clear();
 		bfs::file_status st = di->status(ec);
 		if(ec) {
-			LOG_FS << "Failed to get file status of " << di->path().string() << ": " << ec.message() << '\n';
+			LOG_FS << "Failed to get file status of " << di->path().string() << ": " << ec.message();
 			continue;
 		}
 
@@ -407,15 +464,14 @@ void get_files_in_dir(const std::string& dir,
 			if(checksum != nullptr) {
 				std::time_t mtime = bfs::last_write_time(di->path(), ec);
 				if(ec) {
-					LOG_FS << "Failed to read modification time of " << di->path().string() << ": " << ec.message()
-						   << '\n';
+					LOG_FS << "Failed to read modification time of " << di->path().string() << ": " << ec.message();
 				} else if(mtime > checksum->modified) {
 					checksum->modified = mtime;
 				}
 
 				uintmax_t size = bfs::file_size(di->path(), ec);
 				if(ec) {
-					LOG_FS << "Failed to read filesize of " << di->path().string() << ": " << ec.message() << '\n';
+					LOG_FS << "Failed to read filesize of " << di->path().string() << ": " << ec.message();
 				} else {
 					checksum->sum_size += size;
 				}
@@ -437,10 +493,10 @@ void get_files_in_dir(const std::string& dir,
 			bfs::file_status main_st = bfs::status(inner_main, ec);
 
 			if(error_except_not_found(ec)) {
-				LOG_FS << "Failed to get file status of " << inner_main.string() << ": " << ec.message() << '\n';
+				LOG_FS << "Failed to get file status of " << inner_main.string() << ": " << ec.message();
 			} else if(reorder == reorder_mode::DO_REORDER && main_st.type() == bfs::regular_file) {
 				LOG_FS << "_main.cfg found : "
-					   << (mode == name_mode::ENTIRE_FILE_PATH ? inner_main.string() : inner_main.filename().string()) << '\n';
+					   << (mode == name_mode::ENTIRE_FILE_PATH ? inner_main.string() : inner_main.filename().string());
 				push_if_exists(files, inner_main, mode == name_mode::ENTIRE_FILE_PATH);
 			} else {
 				push_if_exists(dirs, di->path(), mode == name_mode::ENTIRE_FILE_PATH);
@@ -550,12 +606,10 @@ const std::string& get_version_path_suffix()
 
 		if(bfs::is_directory(new_saves_dir)) {
 			if(!bfs::exists(old_saves_dir)) {
-				std::cout << "Apple developer's userdata migration: ";
-				std::cout << "symlinking " << old_saves_dir << " to " << new_saves_dir << "\n";
+				LOG_FS << "Apple developer's userdata migration: symlinking " << old_saves_dir.string() << " to " << new_saves_dir.string();
 				bfs::create_symlink(new_saves_dir, old_saves_dir);
-			} else if(!bfs::symbolic_link_exists(old_saves_dir)) {
-				std::cout << "Apple developer's userdata migration: ";
-				std::cout << "Problem! Old (non-containerized) directory " << old_saves_dir << " is not a symlink. Your savegames are scattered around 2 locations.\n";
+			} else if(!bfs::is_symlink(old_saves_dir)) {
+				ERR_FS << "Apple developer's userdata migration: Problem! Old (non-containerized) directory " << old_saves_dir.string() << " is not a symlink. Your savegames are scattered around 2 locations.";
 			}
 			return;
 		}
@@ -568,28 +622,27 @@ static void setup_user_data_dir()
 #if defined(__APPLE__) && !defined(__IPHONEOS__)
 	migrate_apple_config_directory_for_unsandboxed_builds();
 #endif
-	if(!file_exists(user_data_dir)) {
+	if(!file_exists(user_data_dir / "logs")) {
 		game_config::check_migration = true;
 	}
 
 	if(!create_directory_if_missing_recursive(user_data_dir)) {
-		ERR_FS << "could not open or create user data directory at " << user_data_dir.string() << '\n';
+		ERR_FS << "could not open or create user data directory at " << user_data_dir.string();
 		return;
 	}
 	// TODO: this may not print the error message if the directory exists but we don't have the proper permissions
 
 	// Create user data and add-on directories
-	create_directory_if_missing(user_data_dir / "editor");
-	create_directory_if_missing(user_data_dir / "editor" / "maps");
-	create_directory_if_missing(user_data_dir / "editor" / "scenarios");
-	create_directory_if_missing(user_data_dir / "data");
-	create_directory_if_missing(user_data_dir / "data" / "add-ons");
-	create_directory_if_missing(user_data_dir / "saves");
-	create_directory_if_missing(user_data_dir / "persist");
+	create_directory_if_missing(get_legacy_editor_dir());
+	create_directory_if_missing(get_legacy_editor_dir() + "/maps");
+	create_directory_if_missing(get_legacy_editor_dir() + "/scenarios");
+	create_directory_if_missing(get_addons_data_dir());
+	create_directory_if_missing(get_addons_dir());
+	create_directory_if_missing(get_saves_dir());
+	create_directory_if_missing(get_wml_persist_dir());
+	create_directory_if_missing(get_logs_dir());
 
-#ifdef _WIN32
-	lg::finish_log_file_setup();
-#endif
+	lg::move_log_file();
 }
 
 #ifdef _WIN32
@@ -617,6 +670,11 @@ void set_user_data_dir(std::string newprefdir)
 		newprefdir = PREFERENCES_DIR;
 		relative_ok = true;
 	}
+#elif defined(__APPLE__)
+	if(newprefdir.empty()) {
+		newprefdir = "Library/Application Support/Wesnoth_"+get_version_path_suffix();
+		relative_ok = true;
+	}
 #endif
 
 #ifdef _WIN32
@@ -651,7 +709,7 @@ void set_user_data_dir(std::string newprefdir)
 			//
 			ERR_FS << "Could not determine path to user's Documents folder! (" << std::hex << "0x" << res << std::dec << ") "
 				   << "User config/data directories may be unavailable for "
-				   << "this session. Please report this as a bug.\n";
+				   << "this session. Please report this as a bug.";
 			user_data_dir = bfs::path(get_cwd()) / newprefdir;
 		} else {
 			bfs::path games_path = bfs::path(docs_path) / "My Games";
@@ -714,6 +772,7 @@ void set_user_data_dir(std::string newprefdir)
 #else
 	if(newprefdir.empty()) {
 		newprefdir = backupprefdir;
+		relative_ok = true;
 	}
 
 	const char* home_str = getenv("HOME");
@@ -738,11 +797,23 @@ void set_user_data_dir(std::string newprefdir)
 	user_data_dir = normalize_path(user_data_dir.string(), true, true);
 }
 
+bool rename_dir(const std::string& old_dir, const std::string& new_dir)
+{
+	error_code ec;
+	bfs::rename(old_dir, new_dir, ec);
+
+	if(ec) {
+		ERR_FS << "Failed to rename directory '" << old_dir << "' to '" << new_dir << "'";
+		return false;
+	}
+	return true;
+}
+
 static void set_user_config_path(bfs::path newconfig)
 {
 	user_config_dir = newconfig;
 	if(!create_directory_if_missing_recursive(user_config_dir)) {
-		ERR_FS << "could not open or create user config directory at " << user_config_dir.string() << '\n';
+		ERR_FS << "could not open or create user config directory at " << user_config_dir.string();
 	}
 }
 
@@ -751,6 +822,19 @@ void set_user_config_dir(const std::string& newconfigdir)
 	set_user_config_path(newconfigdir);
 }
 
+static void set_cache_path(bfs::path newcache)
+{
+	cache_dir = newcache;
+	if(!create_directory_if_missing_recursive(cache_dir)) {
+		ERR_FS << "could not open or create cache directory at " << cache_dir.string() << '\n';
+	}
+}
+
+void set_cache_dir(const std::string& newcachedir)
+{
+	set_cache_path(newcachedir);
+}
+
 static const bfs::path& get_user_data_path()
 {
 	if(user_data_dir.empty()) {
@@ -794,6 +878,11 @@ std::string get_user_data_dir()
 	return get_user_data_path().string();
 }
 
+std::string get_logs_dir()
+{
+	return filesystem::get_user_data_dir() + "/logs";
+}
+
 std::string get_cache_dir()
 {
 	if(cache_dir.empty()) {
@@ -882,7 +971,7 @@ std::string get_cwd()
 	bfs::path cwd = bfs::current_path(ec);
 
 	if(ec) {
-		ERR_FS << "Failed to get current directory: " << ec.message() << '\n';
+		ERR_FS << "Failed to get current directory: " << ec.message();
 		return "";
 	}
 
@@ -895,10 +984,10 @@ bool set_cwd(const std::string& dir)
 	bfs::current_path(bfs::path{dir}, ec);
 
 	if(ec) {
-		ERR_FS << "Failed to set current directory: " << ec.message() << '\n';
+		ERR_FS << "Failed to set current directory: " << ec.message();
 		return false;
 	} else {
-		LOG_FS << "Process working directory set to " << dir << '\n';
+		LOG_FS << "Process working directory set to " << dir;
 	}
 
 	return true;
@@ -919,18 +1008,34 @@ std::string get_exe_dir()
 	bfs::path exe(process_path);
 	return exe.parent_path().string();
 #else
+	// first check /proc
 	if(bfs::exists("/proc/")) {
 		bfs::path self_exe("/proc/self/exe");
 		error_code ec;
 		bfs::path exe = bfs::read_symlink(self_exe, ec);
-		if(ec) {
-			return std::string();
+		if(!ec) {
+			return exe.parent_path().string();
 		}
-
-		return exe.parent_path().string();
-	} else {
-		return get_cwd();
 	}
+
+	// check the PATH for wesnoth's location
+	// with version
+	std::string version = std::to_string(game_config::wesnoth_version.major_version()) + "." + std::to_string(game_config::wesnoth_version.minor_version());
+	std::string exe = filesystem::get_program_invocation("wesnoth-"+version);
+	bfs::path search = bp::search_path(exe).string();
+	if(!search.string().empty()) {
+		return search.parent_path().string();
+	}
+
+	// versionless
+	exe = filesystem::get_program_invocation("wesnoth");
+	search = bp::search_path(exe).string();
+	if(!search.string().empty()) {
+		return search.parent_path().string();
+	}
+
+	// return the current working directory
+	return get_cwd();
 #endif
 }
 
@@ -939,7 +1044,7 @@ bool make_directory(const std::string& dirname)
 	error_code ec;
 	bool created = bfs::create_directory(bfs::path(dirname), ec);
 	if(ec) {
-		ERR_FS << "Failed to create directory " << dirname << ": " << ec.message() << '\n';
+		ERR_FS << "Failed to create directory " << dirname << ": " << ec.message();
 	}
 
 	return created;
@@ -958,7 +1063,7 @@ bool delete_directory(const std::string& dirname, const bool keep_pbl)
 		for(const std::string& f : files) {
 			bfs::remove(bfs::path(f), ec);
 			if(ec) {
-				LOG_FS << "remove(" << f << "): " << ec.message() << '\n';
+				LOG_FS << "remove(" << f << "): " << ec.message();
 				ret = false;
 			}
 		}
@@ -976,7 +1081,7 @@ bool delete_directory(const std::string& dirname, const bool keep_pbl)
 	if(ret) {
 		bfs::remove(bfs::path(dirname), ec);
 		if(ec) {
-			LOG_FS << "remove(" << dirname << "): " << ec.message() << '\n';
+			LOG_FS << "remove(" << dirname << "): " << ec.message();
 			ret = false;
 		}
 	}
@@ -989,12 +1094,38 @@ bool delete_file(const std::string& filename)
 	error_code ec;
 	bool ret = bfs::remove(bfs::path(filename), ec);
 	if(ec) {
-		ERR_FS << "Could not delete file " << filename << ": " << ec.message() << '\n';
+		ERR_FS << "Could not delete file " << filename << ": " << ec.message();
 	}
 
 	return ret;
 }
 
+std::vector<uint8_t> read_file_binary(const std::string& fname)
+{
+	std::ifstream file(fname, std::ios::binary);
+	std::vector<uint8_t> file_contents;
+
+	file_contents.reserve(file_size(fname));
+	file_contents.assign(std::istreambuf_iterator<char>(file), std::istreambuf_iterator<char>());
+
+	return file_contents;
+}
+
+std::string read_file_as_data_uri(const std::string& fname)
+{
+	std::vector<uint8_t> file_contents = filesystem::read_file_binary(fname);
+	utils::byte_string_view view = {file_contents.data(), file_contents.size()};
+	std::string name = filesystem::base_name(fname);
+	std::string img = "";
+
+	if(name.find(".") != std::string::npos) {
+		// convert to web-safe base64, since the + symbols will get stripped out when reading this back in later
+		img = "data:image/"+name.substr(name.find(".")+1)+";base64,"+base64::encode(view);
+	}
+
+	return img;
+}
+
 std::string read_file(const std::string& fname)
 {
 	scoped_istream is = istream_file(fname);
@@ -1005,10 +1136,10 @@ std::string read_file(const std::string& fname)
 
 filesystem::scoped_istream istream_file(const std::string& fname, bool treat_failure_as_error)
 {
-	LOG_FS << "Streaming " << fname << " for reading.\n";
+	LOG_FS << "Streaming " << fname << " for reading.";
 
 	if(fname.empty()) {
-		ERR_FS << "Trying to open file with empty name.\n";
+		ERR_FS << "Trying to open file with empty name.";
 		filesystem::scoped_istream s(new bfs::ifstream());
 		s->clear(std::ios_base::failbit);
 		return s;
@@ -1021,9 +1152,9 @@ filesystem::scoped_istream istream_file(const std::string& fname, bool treat_fai
 
 		// TODO: has this still use ?
 		if(!fd.is_open() && treat_failure_as_error) {
-			ERR_FS << "Could not open '" << fname << "' for reading.\n";
+			ERR_FS << "Could not open '" << fname << "' for reading.";
 		} else if(!is_filename_case_correct(fname, fd)) {
-			ERR_FS << "Not opening '" << fname << "' due to case mismatch.\n";
+			ERR_FS << "Not opening '" << fname << "' due to case mismatch.";
 			filesystem::scoped_istream s(new bfs::ifstream());
 			s->clear(std::ios_base::failbit);
 			return s;
@@ -1032,7 +1163,7 @@ filesystem::scoped_istream istream_file(const std::string& fname, bool treat_fai
 		return std::make_unique<boost::iostreams::stream<boost::iostreams::file_descriptor_source>>(fd, 4096, 0);
 	} catch(const std::exception&) {
 		if(treat_failure_as_error) {
-			ERR_FS << "Could not open '" << fname << "' for reading.\n";
+			ERR_FS << "Could not open '" << fname << "' for reading.";
 		}
 
 		filesystem::scoped_istream s(new bfs::ifstream());
@@ -1043,8 +1174,7 @@ filesystem::scoped_istream istream_file(const std::string& fname, bool treat_fai
 
 filesystem::scoped_ostream ostream_file(const std::string& fname, std::ios_base::openmode mode, bool create_directory)
 {
-	LOG_FS << "streaming " << fname << " for writing.\n";
-#if 1
+	LOG_FS << "streaming " << fname << " for writing.";
 	try {
 		boost::iostreams::file_descriptor_sink fd(bfs::path(fname), mode);
 		return std::make_unique<boost::iostreams::stream<boost::iostreams::file_descriptor_sink>>(fd, 4096, 0);
@@ -1058,15 +1188,12 @@ filesystem::scoped_ostream ostream_file(const std::string& fname, std::ios_base:
 
 		throw filesystem::io_exception(e.what());
 	}
-#else
-	return new bfs::ofstream(bfs::path(fname), mode);
-#endif
 }
 
 // Throws io_exception if an error occurs
-void write_file(const std::string& fname, const std::string& data)
+void write_file(const std::string& fname, const std::string& data, std::ios_base::openmode mode)
 {
-	scoped_ostream os = ostream_file(fname);
+	scoped_ostream os = ostream_file(fname, mode);
 	os->exceptions(std::ios_base::goodbit);
 
 	const std::size_t block_size = 4096;
@@ -1113,7 +1240,7 @@ std::time_t file_modified_time(const std::string& fname)
 	error_code ec;
 	std::time_t mtime = bfs::last_write_time(bfs::path(fname), ec);
 	if(ec) {
-		LOG_FS << "Failed to read modification time of " << fname << ": " << ec.message() << '\n';
+		LOG_FS << "Failed to read modification time of " << fname << ": " << ec.message();
 	}
 
 	return mtime;
@@ -1134,7 +1261,7 @@ int file_size(const std::string& fname)
 	error_code ec;
 	uintmax_t size = bfs::file_size(bfs::path(fname), ec);
 	if(ec) {
-		LOG_FS << "Failed to read filesize of " << fname << ": " << ec.message() << '\n';
+		LOG_FS << "Failed to read filesize of " << fname << ": " << ec.message();
 		return -1;
 	} else if(size > INT_MAX) {
 		return INT_MAX;
@@ -1155,7 +1282,7 @@ int dir_size(const std::string& pname)
 	}
 
 	if(ec) {
-		LOG_FS << "Failed to read directorysize of " << pname << ": " << ec.message() << '\n';
+		LOG_FS << "Failed to read directorysize of " << pname << ": " << ec.message();
 		return -1;
 	} else if(size_sum > INT_MAX) {
 		return INT_MAX;
@@ -1325,7 +1452,7 @@ void binary_paths_manager::set_paths(const game_config_view& cfg)
 	for(const config& bp : cfg.child_range("binary_path")) {
 		std::string path = bp["path"].str();
 		if(path.find("..") != std::string::npos) {
-			ERR_FS << "Invalid binary path '" << path << "'\n";
+			ERR_FS << "Invalid binary path '" << path << "'";
 			continue;
 		}
 
@@ -1354,34 +1481,34 @@ void clear_binary_paths_cache()
 
 static bool is_legal_file(const std::string& filename_str)
 {
-	DBG_FS << "Looking for '" << filename_str << "'.\n";
+	DBG_FS << "Looking for '" << filename_str << "'.";
 
 	if(filename_str.empty()) {
-		LOG_FS << "  invalid filename\n";
+		LOG_FS << "  invalid filename";
 		return false;
 	}
 
 	if(filename_str.find("..") != std::string::npos) {
-		ERR_FS << "Illegal path '" << filename_str << "' (\"..\" not allowed).\n";
+		ERR_FS << "Illegal path '" << filename_str << "' (\"..\" not allowed).";
 		return false;
 	}
 
 	if(filename_str.find('\\') != std::string::npos) {
 		ERR_FS << "Illegal path '" << filename_str
-			   << R"end(' ("\" not allowed, for compatibility with GNU/Linux and macOS).)end" << std::endl;
+			   << R"end(' ("\" not allowed, for compatibility with GNU/Linux and macOS).)end";
 		return false;
 	}
 
 	bfs::path filepath(filename_str);
 
 	if(default_blacklist.match_file(filepath.filename().string())) {
-		ERR_FS << "Illegal path '" << filename_str << "' (blacklisted filename)." << std::endl;
+		ERR_FS << "Illegal path '" << filename_str << "' (blacklisted filename).";
 		return false;
 	}
 
 	if(std::any_of(filepath.begin(), filepath.end(),
 			   [](const bfs::path& dirname) { return default_blacklist.match_dir(dirname.string()); })) {
-		ERR_FS << "Illegal path '" << filename_str << "' (blacklisted directory name)." << std::endl;
+		ERR_FS << "Illegal path '" << filename_str << "' (blacklisted directory name).";
 		return false;
 	}
 
@@ -1401,7 +1528,7 @@ const std::vector<std::string>& get_binary_paths(const std::string& type)
 
 	if(type.find("..") != std::string::npos) {
 		// Not an assertion, as language.cpp is passing user data as type.
-		ERR_FS << "Invalid WML type '" << type << "' for binary paths\n";
+		ERR_FS << "Invalid WML type '" << type << "' for binary paths";
 		static std::vector<std::string> dummy;
 		return dummy;
 	}
@@ -1452,20 +1579,20 @@ std::string get_binary_file_location(const std::string& type, const std::string&
 		bfs::path bpath(bp);
 		bpath /= filename;
 
-		DBG_FS << "  checking '" << bp << "'\n";
+		DBG_FS << "  checking '" << bp << "'";
 
 		if(file_exists(bpath)) {
-			DBG_FS << "  found at '" << bpath.string() << "'\n";
+			DBG_FS << "  found at '" << bpath.string() << "'";
 			if(result.empty()) {
 				result = bpath.string();
 			} else {
-				WRN_FS << "Conflicting files in binary_path: '" << sanitize_path(result)
-					   << "' and '" << sanitize_path(bpath.string()) << "'\n";
+				WRN_FS << "Conflicting files in binary_path: '" << result
+					   << "' and '" << bpath.string() << "'";
 			}
 		}
 	}
 
-	DBG_FS << "  not found\n";
+	DBG_FS << "  not found";
 	return result;
 }
 
@@ -1478,14 +1605,14 @@ std::string get_binary_dir_location(const std::string& type, const std::string&
 	for(const std::string& bp : get_binary_paths(type)) {
 		bfs::path bpath(bp);
 		bpath /= filename;
-		DBG_FS << "  checking '" << bp << "'\n";
+		DBG_FS << "  checking '" << bp << "'";
 		if(is_directory_internal(bpath)) {
-			DBG_FS << "  found at '" << bpath.string() << "'\n";
+			DBG_FS << "  found at '" << bpath.string() << "'";
 			return bpath.string();
 		}
 	}
 
-	DBG_FS << "  not found\n";
+	DBG_FS << "  not found";
 	return std::string();
 }
 
@@ -1502,7 +1629,7 @@ std::string get_wml_location(const std::string& filename, const std::string& cur
 
 	if(filename[0] == '~') {
 		result /= get_user_data_path() / "data" / filename.substr(1);
-		DBG_FS << "  trying '" << result.string() << "'\n";
+		DBG_FS << "  trying '" << result.string() << "'";
 	} else if(*fpath.begin() == ".") {
 		if(!current_dir.empty()) {
 			result /= bfs::path(current_dir);
@@ -1516,10 +1643,10 @@ std::string get_wml_location(const std::string& filename, const std::string& cur
 	}
 
 	if(result.empty() || !file_exists(result)) {
-		DBG_FS << "  not found\n";
+		DBG_FS << "  not found";
 		result.clear();
 	} else {
-		DBG_FS << "  found: '" << result.string() << "'\n";
+		DBG_FS << "  found: '" << result.string() << "'";
 	}
 
 	return result.string();
@@ -1593,7 +1720,7 @@ std::string get_program_invocation(const std::string& program_name)
 #endif
 	);
 
-	return (bfs::path(game_config::wesnoth_program_dir) / real_program_name).string();
+	return real_program_name;
 }
 
 std::string sanitize_path(const std::string& path)
@@ -1654,4 +1781,19 @@ std::string get_localized_path(const std::string& file, const std::string& suff)
 	return "";
 }
 
+std::string get_addon_id_from_path(const std::string& location)
+{
+	std::string full_path = normalize_path(location, true);
+	std::string addons_path = normalize_path(get_addons_dir(), true);
+
+	if(full_path.find(addons_path) == 0) {
+		bfs::path path(full_path.substr(addons_path.size()+1));
+		if(path.size() > 0) {
+			return path.begin()->string();
+		}
+	}
+
+	return "";
+}
+
 } // namespace filesystem
