@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2014 - 2022
+	Copyright (C) 2014 - 2024
 	by Nathan Walker <nathan.b.walker@vanderbilt.edu>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -122,9 +122,9 @@ public:
 		return parameters_.shuffle_sides;
 	}
 
-	mp_game_settings::RANDOM_FACTION_MODE random_faction_mode() const
+	random_faction_mode::type mode() const
 	{
-		return parameters_.random_faction_mode;
+		return parameters_.mode;
 	}
 
 	const config& options() const
@@ -231,9 +231,9 @@ public:
 		parameters_.shuffle_sides = val;
 	}
 
-	void set_random_faction_mode(mp_game_settings::RANDOM_FACTION_MODE val)
+	void set_random_faction_mode(random_faction_mode::type val)
 	{
-		parameters_.random_faction_mode = val;
+		parameters_.mode = val;
 	}
 
 	void set_options(const config& cfg);
@@ -258,7 +258,7 @@ public:
 	bool shroud_game_default() const;
 	bool allow_observers_default() const;
 	bool shuffle_sides_default() const;
-	mp_game_settings::RANDOM_FACTION_MODE random_faction_mode_default() const;
+	random_faction_mode::type random_faction_mode_default() const;
 	const config& options_default() const;
 
 	const mp_game_settings& get_parameters() const
