@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -32,7 +32,6 @@
 
 class commandline_options;
 class config;
-class CVideo;
 
 struct jump_to_campaign_info
 {
@@ -122,7 +121,7 @@ private:
 	void start_wesnothd();
 
 	editor::EXIT_STATUS start_editor(const std::string& filename);
-	unit_test_result pass_victory_or_defeat(LEVEL_RESULT res);
+	unit_test_result pass_victory_or_defeat(level_result::type res);
 
 	/**
 	 * Internal to the implementation of unit_test(). If a single instance of
@@ -131,8 +130,7 @@ private:
 	unit_test_result single_unit_test();
 
 	const commandline_options& cmdline_opts_;
-	//Never null.
-	const std::unique_ptr<CVideo> video_;
+	bool start_in_fullscreen_ = false;
 
 	font::manager font_manager_;
 	const preferences::manager prefs_manager_;
