@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2009 - 2022
+	Copyright (C) 2009 - 2024
 	by Mark de Wever <koraq@xs4all.nl>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -57,7 +57,8 @@ public:
 	};
 
 	explicit campaign_selection(ng::create_engine& eng)
-		: engine_(eng)
+		: modal_dialog(window_id())
+		, engine_(eng)
 		, choice_(-1)
 		, rng_mode_(RNG_DEFAULT)
 		, mod_states_()
@@ -66,8 +67,8 @@ public:
 		, current_difficulty_()
 		, current_sorting_(RANK)
 		, currently_sorted_asc_(true)
+		, mod_ids_()
 	{
-		set_restore(true);
 	}
 
 	/***** ***** ***** setters / getters for members ***** ****** *****/
@@ -102,7 +103,7 @@ private:
 
 	void sort_campaigns(CAMPAIGN_ORDER order, bool ascending);
 
-	void add_campaign_to_tree(const config& campaign) const;
+	void add_campaign_to_tree(const config& campaign);
 
 	void toggle_sorting_selection(CAMPAIGN_ORDER order);
 
@@ -131,6 +132,10 @@ private:
 	bool currently_sorted_asc_;
 
 	std::vector<std::string> last_search_words_;
+
+	inline const static std::string missing_campaign_ = "////missing-campaign////";
+
+	std::vector<std::string> mod_ids_;
 };
 
 } // namespace dialogs
