@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2022
+	Copyright (C) 2008 - 2024
 	by Mark de Wever <koraq@xs4all.nl>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -38,7 +38,8 @@ namespace gui2::dialogs
 REGISTER_DIALOG(editor_generate_map)
 
 editor_generate_map::editor_generate_map(std::vector<std::unique_ptr<map_generator>>& mg)
-	: map_generators_(mg)
+	: modal_dialog(window_id())
+	, map_generators_(mg)
 	, last_map_generator_(nullptr)
 	, current_map_generator_(0)
 	, random_seed_()
@@ -86,7 +87,7 @@ void editor_generate_map::pre_show(window& window)
 	listbox& list = find_widget<listbox>(&window, "generators_list", false);
 	window.keyboard_capture(&list);
 
-	std::map<std::string, string_map> lrow;
+	widget_data lrow;
 	for(const auto & gen : map_generators_)
 	{
 		assert(gen);
