@@ -313,12 +313,17 @@ void game_load::display_savegame()
 }
 
 void game_load::filter_text_changed(const std::string& text)
+{
+	apply_filter_text(text, false);
+}
+
+void game_load::apply_filter_text(const std::string& text, bool force)
 {
 	listbox& list = find_widget<listbox>(get_window(), "savegame_list", false);
 
 	const std::vector<std::string> words = utils::split(text, ' ');
 
-	if(words == last_words_)
+	if(words == last_words_ && !force)
 		return;
 	last_words_ = words;
 
@@ -330,13 +335,7 @@ void game_load::filter_text_changed(const std::string& text)
 			bool found = false;
 			for(const auto & word : words)
 			{
-				found = std::search(games_[i].name().begin(),
-									games_[i].name().end(),
-									word.begin(),
-									word.end(),
-									utils::chars_equal_insensitive)
-						!= games_[i].name().end();
-
+				found = translation::ci_search(games_[i].name(), word);
 				if(!found) {
 					// one word doesn't match, we don't reach words.end()
 					break;
@@ -536,6 +535,9 @@ void game_load::handle_dir_select()
 	}
 
 	populate_game_list();
+	if(auto* filter = find_widget<text_box>(get_window(), "txtFilter", false, true)) {
+		apply_filter_text(filter->get_value(), true);
+	}
 	display_savegame();
 }
 
