@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2009 - 2022
+	Copyright (C) 2009 - 2024
 	by Yurii Chernyi <terraninfo@terraninfo.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -34,6 +34,7 @@
 
 #include "game_board.hpp"
 #include "game_data.hpp"
+#include "gettext.hpp"
 #include "recall_list_manager.hpp"
 #include "team.hpp"
 #include "units/unit.hpp"
@@ -42,6 +43,7 @@
 
 #include "display_context.hpp"
 #include "filter_context.hpp"
+#include "video.hpp"
 
 #include <vector>
 #include <functional>
@@ -110,13 +112,13 @@ private:
 	{
 		pages.clear();
 		std::size_t start = 0;
-		while(start + max_inspect_win_len < data.size()) {
+		while(start + page_characters < data.size()) {
 			// This could search into data that's already on a previous page, which is why the result
 			// is then checked for end < start.
-			std::size_t end = data.find_last_of('\n', start + max_inspect_win_len);
+			std::size_t end = data.find_last_of('\n', start + page_characters);
 			int len;
 			if(end == std::string::npos || end < start) {
-				len = max_inspect_win_len;
+				len = page_characters;
 			} else {
 				len = end - start + 1;
 			}
@@ -127,7 +129,7 @@ private:
 			pages.emplace_back(start, data.size() - start);
 		}
 	}
-	static const unsigned int max_inspect_win_len = 20000;
+	unsigned int page_characters = 10000 / video::get_pixel_scale();
 	std::string data;
 	std::vector<std::pair<std::size_t,int>> pages;
 };
@@ -148,7 +150,7 @@ public:
 
 	stuff_list_adder& widget(const std::string& ref, const std::string& label, bool markup = false)
 	{
-		string_map& item = data_[ref];
+		widget_item& item = data_[ref];
 		item["label"] = label;
 		item["use_markup"] = utils::bool_string(markup);
 		return *this;
@@ -157,7 +159,7 @@ public:
 private:
 	tree_view_node& stuff_list_;
 	const std::string defn_;
-	std::map<std::string, string_map> data_;
+	widget_data data_;
 };
 
 class gamestate_inspector::view
@@ -271,6 +273,8 @@ public:
 
 	void show_list(tree_view_node& node);
 	void show_unit(tree_view_node& node);
+	void show_var(tree_view_node& node);
+	void show_array(tree_view_node& node);
 };
 
 class team_mode_controller : public single_mode_controller
@@ -289,6 +293,9 @@ public:
 	void show_recall_unit(tree_view_node& node, int side);
 	void show_units(tree_view_node& node, int side);
 	void show_unit(tree_view_node& node, int side);
+	void show_vars(tree_view_node& node, int side);
+	void show_var(tree_view_node& node, int side);
+	void show_array(tree_view_node& node, int side);
 };
 
 class gamestate_inspector::controller
@@ -492,7 +499,7 @@ const display_context& single_mode_controller::dc() const {
 event_mode_controller::event_mode_controller(gamestate_inspector::controller& c)
 	: single_mode_controller(c)
 {
-	single_mode_controller::events().write_events(events);
+	single_mode_controller::events().write_events(events, true);
 }
 
 void variable_mode_controller::show_list(tree_view_node& node)
@@ -514,7 +521,7 @@ void variable_mode_controller::show_list(tree_view_node& node)
 
 	std::map<std::string, std::size_t> wml_array_sizes;
 
-	for(const auto& ch : vars().all_children_range())
+	for(const auto ch : vars().all_children_range())
 	{
 
 		std::ostringstream cur_str;
@@ -545,7 +552,7 @@ void variable_mode_controller::show_array(tree_view_node& node)
 		std::size_t n_start = var.find_last_of('[') + 1;
 		std::size_t n_len = var.size() - n_start - 1;
 		int n = std::stoi(var.substr(n_start, n_len));
-		model().set_data(config_to_string(vars().child(var.substr(1, n_start - 3), n)));
+		model().set_data(config_to_string(vars().mandatory_child(var.substr(1, n_start - 3), n)));
 	}
 }
 
@@ -580,7 +587,7 @@ void event_mode_controller::show_list(tree_view_node& node, bool is_wmi)
 void event_mode_controller::show_event(tree_view_node& node, bool is_wmi)
 {
 	int n = node.describe_path().back();
-	model().set_data(config_to_string(events.child(is_wmi ? "menu_item" : "event", n)));
+	model().set_data(config_to_string(events.mandatory_child(is_wmi ? "menu_item" : "event", n)));
 }
 
 static stuff_list_adder add_unit_entry(stuff_list_adder& progress, const unit& u, const display_context& dc)
@@ -647,6 +654,61 @@ void unit_mode_controller::show_unit(tree_view_node& node)
 	config c_unit;
 	u->write(c_unit);
 	model().set_data(config_to_string(c_unit));
+
+	if(node.count_children() > 0) {
+		return;
+	}
+
+	for(const auto& attr : u->variables().attribute_range())
+	{
+		c.set_node_callback(
+			view().stuff_list_entry(&node, "basic")
+				.widget("name", attr.first)
+				.add(),
+			&unit_mode_controller::show_var);
+	}
+
+	std::map<std::string, std::size_t> wml_array_sizes;
+
+	for(const auto ch : u->variables().all_children_range())
+	{
+
+		std::ostringstream cur_str;
+		cur_str << "[" << ch.key << "][" << wml_array_sizes[ch.key] << "]";
+
+		this->c.set_node_callback(
+			view().stuff_list_entry(&node, "basic")
+				.widget("name", cur_str.str())
+				.add(),
+			&unit_mode_controller::show_array);
+		wml_array_sizes[ch.key]++;
+	}
+}
+
+void unit_mode_controller::show_var(tree_view_node& node)
+{
+	widget* w = node.find("name", false);
+	int i = node.describe_path().back();
+	unit_map::const_iterator u = dc().units().begin();
+	std::advance(u, i);
+	if(label* lbl = dynamic_cast<label*>(w)) {
+		model().set_data(u->variables()[lbl->get_label().str()]);
+	}
+}
+
+void unit_mode_controller::show_array(tree_view_node& node)
+{
+	widget* w = node.find("name", false);
+	int i = node.describe_path().back();
+	unit_map::const_iterator u = dc().units().begin();
+	std::advance(u, i);
+	if(label* lbl = dynamic_cast<label*>(w)) {
+		const std::string& var = lbl->get_label();
+		std::size_t n_start = var.find_last_of('[') + 1;
+		std::size_t n_len = var.size() - n_start - 1;
+		int n = std::stoi(var.substr(n_start, n_len));
+		model().set_data(config_to_string(u->variables().mandatory_child(var.substr(1, n_start - 3), n)));
+	}
 }
 
 void team_mode_controller::show_list(tree_view_node& node, int side)
@@ -677,6 +739,12 @@ void team_mode_controller::show_list(tree_view_node& node, int side)
 			.add(),
 		&team_mode_controller::show_units,
 		side);
+	c.set_node_callback(
+		view().stuff_list_entry(&node, "basic")
+			.widget("name", "variables")
+			.add(),
+		&team_mode_controller::show_vars,
+		side);
 }
 
 void team_mode_controller::show_ai(tree_view_node& node, int side)
@@ -785,10 +853,71 @@ void team_mode_controller::show_units(tree_view_node&, int side)
 	model().set_data(s.str());
 }
 
+void team_mode_controller::show_vars(tree_view_node& node, int side)
+{
+	model().clear_data();
+
+	if(node.count_children() > 0) {
+		return;
+	}
+
+	const team& t = dc().get_team(side);
+
+	for(const auto& attr : t.variables().attribute_range())
+	{
+		c.set_node_callback(
+			view().stuff_list_entry(&node, "basic")
+				.widget("name", attr.first)
+				.add(),
+			&team_mode_controller::show_var,
+			side);
+	}
+
+	std::map<std::string, std::size_t> wml_array_sizes;
+
+	for(const auto ch : t.variables().all_children_range())
+	{
+
+		std::ostringstream cur_str;
+		cur_str << "[" << ch.key << "][" << wml_array_sizes[ch.key] << "]";
+
+		this->c.set_node_callback(
+			view().stuff_list_entry(&node, "basic")
+				.widget("name", cur_str.str())
+				.add(),
+			&team_mode_controller::show_array,
+			side);
+		wml_array_sizes[ch.key]++;
+	}
+}
+
+void team_mode_controller::show_var(tree_view_node& node, int side)
+{
+	widget* w = node.find("name", false);
+	const team& t = dc().get_team(side);
+	if(label* lbl = dynamic_cast<label*>(w)) {
+		model().set_data(t.variables()[lbl->get_label().str()]);
+	}
+}
+
+void team_mode_controller::show_array(tree_view_node& node, int side)
+{
+	widget* w = node.find("name", false);
+	const team& t = dc().get_team(side);
+	if(label* lbl = dynamic_cast<label*>(w)) {
+		const std::string& var = lbl->get_label();
+		std::size_t n_start = var.find_last_of('[') + 1;
+		std::size_t n_len = var.size() - n_start - 1;
+		int n = std::stoi(var.substr(n_start, n_len));
+		model().set_data(config_to_string(t.variables().mandatory_child(var.substr(1, n_start - 3), n)));
+	}
+}
+
 REGISTER_DIALOG(gamestate_inspector)
 
 gamestate_inspector::gamestate_inspector(const config& vars, const game_events::manager& events, const display_context& dc, const std::string& title)
-	: title_(title)
+	: modal_dialog(window_id())
+	, title_(title)
 	, vars_(vars)
 	, events_(events)
 	, dc_(dc)
