@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2017 - 2022
+	Copyright (C) 2017 - 2024
 	by Charles Dang <exodia339@gmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -17,6 +17,7 @@
 
 #include "gui/dialogs/story_viewer.hpp"
 
+#include "display.hpp"
 #include "formula/variant.hpp"
 #include "gui/auxiliary/find_widget.hpp"
 #include "sdl/point.hpp"
@@ -54,7 +55,8 @@ static const unsigned int LAYER_TEXT = 2;
 REGISTER_DIALOG(story_viewer)
 
 story_viewer::story_viewer(const std::string& scenario_name, const config& cfg_parsed)
-	: controller_(vconfig(cfg_parsed, true), scenario_name)
+	: modal_dialog(window_id())
+	, controller_(vconfig(cfg_parsed, true), scenario_name)
 	, part_index_(0)
 	, current_part_(nullptr)
 	, timer_id_(0)
@@ -91,12 +93,19 @@ void story_viewer::pre_show(window& window)
 	connect_signal_mouse_left_click(find_widget<button>(&window, "back", false),
 		std::bind(&story_viewer::nav_button_callback, this, DIR_BACKWARDS));
 
-	connect_signal_on_draw(window,
-		std::bind(&story_viewer::draw_callback, this));
+	// Tell the game display not to draw
+	game_was_already_hidden_ = display::get_singleton()->get_prevent_draw();
+	display::get_singleton()->set_prevent_draw(true);
 
 	display_part();
 }
 
+void story_viewer::post_show(window& /*window*/)
+{
+	// Bring the game display back again, if appropriate
+	display::get_singleton()->set_prevent_draw(game_was_already_hidden_);
+}
+
 void story_viewer::update_current_part_ptr()
 {
 	current_part_ = controller_.get_part(part_index_);
@@ -223,8 +232,8 @@ void story_viewer::display_part()
 	window_canvas.set_cfg(cfg);
 
 	// Needed to make the background redraw correctly.
-	window_canvas.set_is_dirty(true);
-	get_window()->set_is_dirty(true);
+	window_canvas.update_size_variables();
+	get_window()->queue_redraw();
 
 	//
 	// Title
@@ -298,6 +307,9 @@ void story_viewer::display_part()
 	text_label.set_text_alpha(0);
 	text_label.set_label(part_text);
 
+	// Regenerate any background blur texture
+	panel_canvas.queue_reblur();
+
 	begin_fade_draw(true);
 	// if the previous page was skipped, it is possible that we already have a timer running.
 	clear_image_timer();
@@ -358,8 +370,8 @@ void story_viewer::draw_floating_image(floating_image_list::const_iterator image
 		window_canvas.append_cfg(std::move(cfg));
 
 		// Needed to make the background redraw correctly.
-		window_canvas.set_is_dirty(true);
-		get_window()->set_is_dirty(true);
+		window_canvas.update_size_variables();
+		get_window()->queue_redraw();
 
 		// If a delay is specified, schedule the next image draw and break out of the loop.
 		const unsigned int draw_delay = floating_image.display_delay();
@@ -451,8 +463,10 @@ void story_viewer::halt_fade_draw()
 	fade_state_ = NOT_FADING;
 }
 
-void story_viewer::draw_callback()
+void story_viewer::update()
 {
+	modal_dialog::update();
+
 	if(next_draw_ && SDL_GetTicks() < next_draw_) {
 		return;
 	}
@@ -492,7 +506,7 @@ void story_viewer::draw_callback()
 
 void story_viewer::flag_stack_as_dirty()
 {
-	find_widget<stacked_widget>(get_window(), "text_and_control_stack", false).set_is_dirty(true);
+	find_widget<stacked_widget>(get_window(), "text_and_control_stack", false).queue_redraw();
 }
 
 } // namespace dialogs
