@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2016 - 2022
+	Copyright (C) 2016 - 2024
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
 	This program is free software; you can redistribute it and/or modify
@@ -17,15 +17,12 @@
 #include "gui/widgets/chatbox.hpp"
 
 #include "gui/auxiliary/find_widget.hpp"
-
 #include "gui/core/register_widget.hpp"
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/image.hpp"
-#include "gui/widgets/label.hpp"
 #include "gui/widgets/listbox.hpp"
 #include "gui/widgets/multi_page.hpp"
 #include "gui/widgets/scroll_label.hpp"
-#include "gui/widgets/settings.hpp"
 #include "gui/widgets/text_box.hpp"
 #include "gui/widgets/window.hpp"
 
@@ -39,7 +36,7 @@
 #include "preferences/game.hpp"
 #include "preferences/lobby.hpp"
 #include "scripting/plugins/manager.hpp"
-#include "wesnothd_connection.hpp"
+#include "wml_exception.hpp"
 
 static lg::log_domain log_lobby("lobby");
 #define DBG_LB LOG_STREAM(debug, log_lobby)
@@ -63,8 +60,6 @@ chatbox::chatbox(const implementation::builder_chatbox& builder)
 	, chat_input_(nullptr)
 	, active_window_(0)
 	, active_window_changed_callback_()
-	, chat_info_()
-	, wesnothd_connection_(nullptr)
 	, log_(nullptr)
 {
 	// We only implement a RECEIVE_KEYBOARD_FOCUS handler; LOSE_KEYBOARD_FOCUS
@@ -301,8 +296,6 @@ void chatbox::add_whisper_sent(const std::string& receiver, const std::string& m
 	} else {
 		add_active_window_whisper(VGETTEXT("whisper to $receiver", {{"receiver", receiver}}), message, true);
 	}
-
-	chat_info_.get_whisper_log(receiver).add_message(preferences::login(), message);
 }
 
 void chatbox::add_whisper_received(const std::string& sender, const std::string& message)
@@ -310,24 +303,22 @@ void chatbox::add_whisper_received(const std::string& sender, const std::string&
 	bool can_go_to_active = !preferences::whisper_friends_only() || preferences::is_friend(sender);
 	bool can_open_new = preferences::auto_open_whisper_windows() && can_go_to_active;
 
-	chat_info_.get_whisper_log(sender).add_message(sender, message);
-
 	if(whisper_window_open(sender, can_open_new)) {
 		if(whisper_window_active(sender)) {
 			add_active_window_message(sender, message);
 
-			do_notify(mp::NOTIFY_WHISPER, sender, message);
+			do_notify(mp::notify_mode::whisper, sender, message);
 		} else {
 			add_whisper_window_whisper(sender, message);
 			increment_waiting_whispers(sender);
 
-			do_notify(mp::NOTIFY_WHISPER_OTHER_WINDOW, sender, message);
+			do_notify(mp::notify_mode::whisper_other_window, sender, message);
 		}
 	} else if(can_go_to_active) {
 		add_active_window_whisper(sender, message);
-		do_notify(mp::NOTIFY_WHISPER, sender, message);
+		do_notify(mp::notify_mode::whisper, sender, message);
 	} else {
-		LOG_LB << "Ignoring whisper from " << sender << "\n";
+		LOG_LB << "Ignoring whisper from " << sender;
 	}
 }
 
@@ -335,19 +326,14 @@ void chatbox::add_chat_room_message_sent(const std::string& room, const std::str
 {
 	lobby_chat_window* t = room_window_open(room, false);
 	if(!t) {
-		LOG_LB << "Cannot add sent message to ui for room " << room << ", player not in the room\n";
+		LOG_LB << "Cannot add sent message to ui for room " << room << ", player not in the room";
 		return;
 	}
 
-	// Do not open room window here. The player should be in the room before sending messages
-	mp::room_info* ri = chat_info_.get_room(room);
-	assert(ri);
-
 	if(!room_window_active(room)) {
 		switch_to_window(t);
 	}
 
-	ri->log().add_message(preferences::login(), message);
 	add_active_window_message(preferences::login(), message, true);
 }
 
@@ -355,30 +341,23 @@ void chatbox::add_chat_room_message_received(const std::string& room,
 	const std::string& speaker,
 	const std::string& message)
 {
-	mp::room_info* ri = chat_info_.get_room(room);
-	if(!ri) {
-		LOG_LB << "Discarding message to room " << room << " from " << speaker << " (room not open)\n";
-		return;
-	}
-
-	mp::notify_mode notify_mode = mp::NOTIFY_NONE;
-	ri->log().add_message(speaker, message);
+	mp::notify_mode notify_mode = mp::notify_mode::none;
 
 	if(room_window_active(room)) {
 		add_active_window_message(speaker, message);
-		notify_mode = mp::NOTIFY_MESSAGE;
+		notify_mode = mp::notify_mode::message;
 	} else {
 		add_room_window_message(room, speaker, message);
 		increment_waiting_messages(room);
-		notify_mode = mp::NOTIFY_MESSAGE_OTHER_WINDOW;
+		notify_mode = mp::notify_mode::message_other_window;
 	}
 
 	if(speaker == "server") {
-		notify_mode = mp::NOTIFY_SERVER_MESSAGE;
+		notify_mode = mp::notify_mode::server_message;
 	} else if (utils::word_match(message, preferences::login())) {
-		notify_mode = mp::NOTIFY_OWN_NICK;
+		notify_mode = mp::notify_mode::own_nick;
 	} else if (preferences::is_friend(speaker)) {
-		notify_mode = mp::NOTIFY_FRIEND_MESSAGE;
+		notify_mode = mp::notify_mode::friend_message;
 	}
 
 	do_notify(notify_mode, speaker, message);
@@ -430,14 +409,10 @@ lobby_chat_window* chatbox::find_or_create_window(const std::string& name,
 	//
 	// Add a new chat log page.
 	//
-	string_map item;
+	widget_item item;
 	item["use_markup"] = "true";
 	item["label"] = initial_text;
-	std::map<std::string, string_map> data{{"log_text", item}};
-
-	if(!whisper) {
-		chat_info_.open_room(name);
-	}
+	widget_data data{{"log_text", item}};
 
 	if(log_ != nullptr) {
 		log_->emplace(name, chatroom_log{item["label"], whisper});
@@ -489,9 +464,7 @@ void chatbox::close_window_button_callback(std::string room_name, bool& handled,
 
 void chatbox::send_to_server(const ::config& cfg)
 {
-	if(wesnothd_connection_) {
-		wesnothd_connection_->send_data(cfg);
-	}
+	mp::send_to_server(cfg);
 }
 
 void chatbox::increment_waiting_whispers(const std::string& name)
@@ -500,7 +473,7 @@ void chatbox::increment_waiting_whispers(const std::string& name)
 		++t->pending_messages;
 
 		if(t->pending_messages == 1) {
-			DBG_LB << "do whisper pending mark row " << (t - &open_windows_[0]) << " with " << t->name << "\n";
+			DBG_LB << "do whisper pending mark row " << (t - &open_windows_[0]) << " with " << t->name;
 
 			grid* grid = roomlistbox_->get_row_grid(t - &open_windows_[0]);
 			find_widget<image>(grid, "pending_messages", false).set_visible(widget::visibility::visible);
@@ -516,7 +489,7 @@ void chatbox::increment_waiting_messages(const std::string& room)
 		if(t->pending_messages == 1) {
 			int idx = t - &open_windows_[0];
 
-			DBG_LB << "do room pending mark row " << idx << " with " << t->name << "\n";
+			DBG_LB << "do room pending mark row " << idx << " with " << t->name;
 
 			grid* grid = roomlistbox_->get_row_grid(idx);
 			find_widget<image>(grid, "pending_messages", false).set_visible(widget::visibility::visible);
@@ -528,7 +501,7 @@ void chatbox::add_whisper_window_whisper(const std::string& sender, const std::s
 {
 	lobby_chat_window* t = whisper_window_open(sender, false);
 	if(!t) {
-		ERR_LB << "Whisper window not open in add_whisper_window_whisper for " << sender << "\n";
+		ERR_LB << "Whisper window not open in add_whisper_window_whisper for " << sender;
 		return;
 	}
 
@@ -548,7 +521,7 @@ void chatbox::close_window(std::size_t idx)
 {
 	const lobby_chat_window& t = open_windows_[idx];
 
-	DBG_LB << "Close window " << idx << " - " << t.name << "\n";
+	DBG_LB << "Close window " << idx << " - " << t.name;
 
 	// Can't close the lobby!
 	if((t.name == "lobby" && t.whisper == false) || open_windows_.size() == 1) {
@@ -562,12 +535,6 @@ void chatbox::close_window(std::size_t idx)
 		--active_window_;
 	}
 
-	if(t.whisper) {
-		chat_info_.get_whisper_log(t.name).clear();
-	} else {
-		chat_info_.close_room(t.name);
-	}
-
 	if(log_ != nullptr) {
 		log_->erase(t.name);
 	}
@@ -591,7 +558,7 @@ void chatbox::add_room_window_message(const std::string& room,
 {
 	lobby_chat_window* t = room_window_open(room, false);
 	if(!t) {
-		ERR_LB << "Room window not open in add_room_window_message for " << room << "\n";
+		ERR_LB << "Room window not open in add_room_window_message for " << room;
 		return;
 	}
 
@@ -607,21 +574,11 @@ void chatbox::add_active_window_message(const std::string& sender,
 	append_to_chatbox(text, force_scroll);
 }
 
-mp::room_info* chatbox::active_window_room()
-{
-	const lobby_chat_window& t = open_windows_[active_window_];
-	if(t.whisper) {
-		return nullptr;
-	}
-
-	return chat_info_.get_room(t.name);
-}
-
 void chatbox::process_message(const ::config& data, bool whisper /*= false*/)
 {
 	std::string sender = data["sender"];
 	DBG_LB << "process message from " << sender << " " << (whisper ? "(w)" : "")
-		<< ", len " << data["message"].str().size() << '\n';
+		<< ", len " << data["message"].str().size();
 
 	if(preferences::is_ignored(sender)) {
 		return;
@@ -639,19 +596,19 @@ void chatbox::process_message(const ::config& data, bool whisper /*= false*/)
 
 		// Attempt to send to the currently active room first.
 		if(room.empty()) {
-			LOG_LB << "Message without a room from " << sender << ", falling back to active window\n";
+			LOG_LB << "Message without a room from " << sender << ", falling back to active window";
 			room = open_windows_[active_window_].name;
 		}
 
 		// If we still don't have a name, fall back to lobby.
 		if(room.empty()) {
-			LOG_LB << "Message without a room from " << sender << ", assuming lobby\n";
+			LOG_LB << "Message without a room from " << sender << ", assuming lobby";
 			room = "lobby";
 		}
 
 		if(log_ != nullptr && data["type"].str() == "motd") {
 			if(log_->at("lobby").received_motd == message) {
-				LOG_LB << "Ignoring repeated motd\n";
+				LOG_LB << "Ignoring repeated motd";
 				return;
 			} else {
 				log_->at("lobby").received_motd = message;
@@ -669,16 +626,16 @@ void chatbox::process_message(const ::config& data, bool whisper /*= false*/)
 
 void chatbox::process_network_data(const ::config& data)
 {
-	if(const ::config& message = data.child("message")) {
-		process_message(message);
-	} else if(const ::config& whisper = data.child("whisper")) {
-		process_message(whisper, true);
+	if(const auto message = data.optional_child("message")) {
+		process_message(*message);
+	} else if(const auto whisper = data.optional_child("whisper")) {
+		process_message(*whisper, true);
 	}
 }
 
 void chatbox::signal_handler_receive_keyboard_focus(const event::ui_event event)
 {
-	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".";
 
 	// Forward focus to the input textbox.
 	get_window()->keyboard_capture(chat_input_);
@@ -695,12 +652,10 @@ chatbox_definition::chatbox_definition(const config& cfg)
 chatbox_definition::resolution::resolution(const config& cfg)
 	: resolution_definition(cfg), grid()
 {
-	state.emplace_back(cfg.child("background"));
-	state.emplace_back(cfg.child("foreground"));
-
-	const config& child = cfg.child("grid");
-	VALIDATE(child, _("No grid defined."));
+	state.emplace_back(VALIDATE_WML_CHILD(cfg, "background", missing_mandatory_wml_tag("chatbox_definition][resolution", "background")));
+	state.emplace_back(VALIDATE_WML_CHILD(cfg, "foreground", missing_mandatory_wml_tag("chatbox_definition][resolution", "foreground")));
 
+	auto child = VALIDATE_WML_CHILD(cfg, "grid", missing_mandatory_wml_tag("chatbox_definition][resolution", "grid"));
 	grid = std::make_shared<builder_grid>(child);
 }
 // }---------- BUILDER -----------{
@@ -713,12 +668,12 @@ builder_chatbox::builder_chatbox(const config& cfg)
 {
 }
 
-widget* builder_chatbox::build() const
+std::unique_ptr<widget> builder_chatbox::build() const
 {
-	chatbox* widget = new chatbox(*this);
+	auto widget = std::make_unique<chatbox>(*this);
 
 	DBG_GUI_G << "Window builder: placed unit preview pane '" << id
-			  << "' with definition '" << definition << "'.\n";
+			  << "' with definition '" << definition << "'.";
 
 	const auto conf = widget->cast_config_to<chatbox_definition>();
 	assert(conf);
