@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2021
+	Copyright (C) 2008 - 2024
 	by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -17,7 +17,6 @@
 
 #include "serialization/base64.hpp"
 
-#include <iostream>
 #include <string>
 #include <sstream>
 #include <string.h>
@@ -29,18 +28,13 @@ extern "C" {
 
 #ifndef __APPLE__
 
-#include <openssl/sha.h>
-#include <openssl/md5.h>
-
-static_assert(utils::md5::DIGEST_SIZE == MD5_DIGEST_LENGTH, "Constants mismatch");
-static_assert(utils::sha1::DIGEST_SIZE == SHA_DIGEST_LENGTH, "Constants mismatch");
+#include <openssl/evp.h>
 
 #else
 
 #include <CommonCrypto/CommonDigest.h>
 
 static_assert(utils::md5::DIGEST_SIZE == CC_MD5_DIGEST_LENGTH, "Constants mismatch");
-static_assert(utils::sha1::DIGEST_SIZE == CC_SHA1_DIGEST_LENGTH, "Constants mismatch");
 
 #endif
 
@@ -71,10 +65,19 @@ namespace utils {
 md5::md5(const std::string& input) {
 
 #ifndef __APPLE__
-	MD5_CTX md5_worker;
-	MD5_Init(&md5_worker);
-	MD5_Update(&md5_worker, input.data(), input.size());
-	MD5_Final(hash.data(), &md5_worker);
+	EVP_MD_CTX *mdctx = EVP_MD_CTX_new();
+	unsigned int md5_digest_len = EVP_MD_size(EVP_md5());
+	assert(utils::md5::DIGEST_SIZE == md5_digest_len);
+
+	// MD5_Init
+	EVP_DigestInit_ex(mdctx, EVP_md5(), NULL);
+
+	// MD5_Update
+	EVP_DigestUpdate(mdctx, input.c_str(), input.size());
+
+	// MD5_Final
+	EVP_DigestFinal_ex(mdctx, hash.data(), &md5_digest_len);
+	EVP_MD_CTX_free(mdctx);
 #else
 	CC_MD5(input.data(), static_cast<CC_LONG>(input.size()), hash.data());
 #endif
@@ -124,28 +127,6 @@ std::string md5::base64_digest() const
 	return encode_hash<DIGEST_SIZE>(hash);
 }
 
-sha1::sha1(const std::string& str)
-{
-#ifndef __APPLE__
-	SHA_CTX hasher;
-	SHA1_Init(&hasher);
-	SHA1_Update(&hasher, str.data(), str.size());
-	SHA1_Final(hash.data(), &hasher);
-#else
-	CC_MD5(str.data(), static_cast<CC_LONG>(str.size()), hash.data());
-#endif
-}
-
-std::string sha1::hex_digest() const
-{
-	return hexencode_hash<DIGEST_SIZE>(hash);
-}
-
-std::string sha1::base64_digest() const
-{
-	return encode_hash<DIGEST_SIZE>(hash);
-}
-
 bcrypt::bcrypt(const std::string& input)
 {
 	assert(is_valid_prefix(input));
