@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -32,11 +32,10 @@
 #include "tstring.hpp"                  // for t_string, operator<<
 #include "units/helper.hpp"             // for resistance_color
 #include "units/types.hpp"              // for unit_type, unit_type_data, etc
-#include <optional>
-#include "video.hpp"                    // fore current_resolution
+#include "video.hpp"                    // for game_canvas_size
 
-#include <iostream>                     // for operator<<, basic_ostream, etc
 #include <map>                          // for map, etc
+#include <optional>
 #include <set>
 
 static lg::log_domain log_help("help");
@@ -150,7 +149,7 @@ std::string terrain_topic_generator::operator()() const {
 	std::shared_ptr<terrain_type_data> tdata = load_terrain_types_data();
 
 	if (!tdata) {
-		WRN_HP << "When building terrain help topics, we couldn't acquire any terrain types data\n";
+		WRN_HP << "When building terrain help topics, we couldn't acquire any terrain types data";
 		return ss.str();
 	}
 
@@ -293,7 +292,7 @@ std::string unit_topic_generator::operator()() const {
 	const unit_type& female_type = type_.get_gender_unit_type(unit_race::FEMALE);
 	const unit_type& male_type = type_.get_gender_unit_type(unit_race::MALE);
 
-	const int screen_width = CVideo::get_singleton().get_width();
+	const int screen_width = video::game_canvas_size().x;
 
 	ss << _("Level") << " " << type_.level();
 	ss << "\n\n";
@@ -303,8 +302,7 @@ std::string unit_topic_generator::operator()() const {
 	if (screen_width >= 1200) ss << "~XBRZ(2)";
 	ss << "' box='no'</img> ";
 
-
-	if (&female_type != &male_type) {
+	if (female_type.image() != male_type.image()) {
 		ss << "<img>src='" << female_type.image();
 		ss << "~RC(" << female_type.flag_rgb() << ">red)";
 		if (screen_width >= 1200) ss << "~XBRZ(2)";
@@ -326,6 +324,10 @@ std::string unit_topic_generator::operator()() const {
 		sz = 400;
 	}
 
+	// without this, scaling down (SCALE_INTO below) and then scaling back up due to the pixel multiplier leads to ugly results
+	// can't use the preferences value since it may be different than the actual value
+	sz *= video::get_pixel_scale();
+
 	// TODO: figure out why the second checks don't match but the last does
 	if (has_male_portrait) {
 		ss << "<img>src='" << male_portrait << "~FL(horiz)~SCALE_INTO(" << sz << ',' << sz << ")' box='no' align='right' float='yes'</img> ";
@@ -605,6 +607,9 @@ std::string unit_topic_generator::operator()() const {
 		}
 	}
 
+	// Padding for range and damage type icons
+	const auto padding = 4; // matches the alignment of the terrain rows
+
 	// Print the different attacks a unit has, if it has any.
 	if (!type_.attacks().empty()) {
 		// Print headers for the table.
@@ -642,11 +647,8 @@ std::string unit_topic_generator::operator()() const {
 			push_tab_pair(row, attack_ss.str());
 			attack_ss.str(clear_stringstream);
 
-			// Padding for range and damage type icons
-			const auto padding = 5; // TODO amount of padding?
-
 			// Range, with icon
-			const std::string range_icon = "icons/profiles/" + attack.range() + "_attack.png";
+			const std::string range_icon = "icons/profiles/" + attack.range() + "_attack.png~SCALE_INTO(16,16)";
 			if (attack.min_range() > 1 || attack.max_range() > 1) {
 				attack_ss << attack.min_range() << "-" << attack.max_range() << ' ';
 			}
@@ -655,7 +657,7 @@ std::string unit_topic_generator::operator()() const {
 			attack_ss.str(clear_stringstream);
 
 			// Damage type, with icon
-			const std::string type_icon = "icons/profiles/" + attack.type() + ".png";
+			const std::string type_icon = "icons/profiles/" + attack.type() + ".png~SCALE_INTO(16,16)";
 			push_tab_pair(row, lang_type, type_icon, padding);
 
 			// Show this attack's special, if it has any. Cross
@@ -687,7 +689,7 @@ std::string unit_topic_generator::operator()() const {
 		for (const config & t : traits) {
 			if (t["availability"].str() == "musthave") {
 				for (const config & effect : t.child_range("effect")) {
-					if (!effect.child("filter") // If this is musthave but has a unit filter, it might not always apply, so don't apply it in the help.
+					if (!effect.has_child("filter") // If this is musthave but has a unit filter, it might not always apply, so don't apply it in the help.
 							&& movetype::effects.find(effect["apply_to"].str()) != movetype::effects.end()) {
 						movement_type.merge(effect, effect["replace"].to_bool());
 					}
@@ -704,7 +706,7 @@ std::string unit_topic_generator::operator()() const {
 	push_header(first_res_row, _("Attack Type"));
 	push_header(first_res_row, _("Resistance"));
 	resistance_table.push_back(first_res_row);
-	utils::string_map dam_tab = movement_type.damage_table();
+	utils::string_map_res dam_tab = movement_type.damage_table();
 	for(std::pair<std::string, std::string> dam_it : dam_tab) {
 		std::vector<item> row;
 		int resistance = 100;
@@ -717,8 +719,9 @@ std::string unit_topic_generator::operator()() const {
 			resist.replace(pos, 1, font::unicode_minus);
 		}
 		std::string color = unit_helper::resistance_color(resistance);
-		std::string lang_weapon = string_table["type_" + dam_it.first];
-		push_tab_pair(row, lang_weapon);
+		const std::string lang_type = string_table["type_" + dam_it.first];
+		const std::string type_icon = "icons/profiles/" + dam_it.first + ".png~SCALE_INTO(16,16)";
+		push_tab_pair(row, lang_type, type_icon, padding);
 		std::stringstream str;
 		str << "<format>color=\"" << color << "\" text='"<< resist << "'</format>";
 		const std::string markup = str.str();
@@ -803,107 +806,114 @@ std::string unit_topic_generator::operator()() const {
 			std::string color = game_config::red_to_green(m.defense, false).to_hex_string();
 
 			std::stringstream str;
+			std::stringstream str_unformatted;
 			str << "<format>color='" << color << "' text='"<< m.defense << "%'</format>";
-			std::string markup = str.str();
-			str.str(clear_stringstream);
-			str << m.defense << "%";
-			row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
+			str_unformatted << m.defense << "%";
+			row.emplace_back(str.str(), font::pango_line_width(str_unformatted.str(), normal_font_size));
 
 			//movement  -  range: 1 .. 5, movetype::UNREACHABLE=impassable
 			str.str(clear_stringstream);
-			bool cannot_move = m.movement_cost > type_.movement();
-			if (cannot_move) {		// cannot move in this terrain
-				color = "red";
-			} else if (m.movement_cost > 1) {
-				color = "yellow";
-			} else {
-				color = "white";
-			}
-			str << "<format>color=" << color << " text='";
+			str_unformatted.str(clear_stringstream);
+			const bool cannot_move = m.movement_cost > type_.movement();        // cannot move in this terrain
+			double movement_red_to_green = 100.0 - 25.0 * m.movement_cost;
+
+			// passing true to select the less saturated red-to-green scale
+			std::string movement_color = game_config::red_to_green(movement_red_to_green, true).to_hex_string();
+			str << "<format>color='" << movement_color << "' text='";
 			// A 5 MP margin; if the movement costs go above
 			// the unit's max moves + 5, we replace it with dashes.
 			if(cannot_move && (m.movement_cost > type_.movement() + 5)) {
-				str << font::unicode_figure_dash;
+				str_unformatted << font::unicode_figure_dash;
+			} else if(cannot_move) {
+				str_unformatted << "(" << m.movement_cost << ")";
 			} else {
-				str << m.movement_cost;
+				str_unformatted << m.movement_cost;
 			}
-			str << "'</format>";
-			markup = str.str();
-			str.str(clear_stringstream);
-			str << m.movement_cost;
-			row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
+			if(m.movement_cost != 0) {
+				const int movement_hexes_per_turn = type_.movement() / m.movement_cost;
+				str_unformatted << " ";
+				for(int i = 0; i < movement_hexes_per_turn; ++i) {
+					// Unicode horizontal black hexagon and Unicode zero width space (to allow a line break)
+					str_unformatted << "\u2b23\u200b";
+				}
+			}
+			str << str_unformatted.str() << "'</format>";
+			row.emplace_back(str.str(), font::pango_line_width(str_unformatted.str(), normal_font_size));
 
 			//defense cap
 			if (has_terrain_defense_caps) {
 				str.str(clear_stringstream);
+				str_unformatted.str(clear_stringstream);
 				if (m.defense_cap) {
 					str << "<format>color='"<< color <<"' text='" << m.defense << "%'</format>";
+					str_unformatted << m.defense << "%";
 				} else {
 					str << "<format>color=white text='" << font::unicode_figure_dash << "'</format>";
+					str_unformatted << font::unicode_figure_dash;
 				}
-
-				markup = str.str();
-				str.str(clear_stringstream);
-				if (m.defense_cap) {
-					str << m.defense << '%';
-				} else {
-					str << font::unicode_figure_dash;
-				}
-				row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
+				row.emplace_back(str.str(), font::pango_line_width(str_unformatted.str(), normal_font_size));
 			}
 
 			//vision
 			if (has_vision) {
 				str.str(clear_stringstream);
-				const bool cannot_view = m.vision_cost > type_.vision();
-				if (cannot_view) {		// cannot view in this terrain
-					color = "red";
-				} else if (m.vision_cost > m.movement_cost) {
-					color = "yellow";
-				} else if (m.vision_cost == m.movement_cost) {
-					color = "white";
-				} else {
-					color = "green";
-				}
-				str << "<format>color=" << color << " text='";
+				str_unformatted.str(clear_stringstream);
+				const bool cannot_view = m.vision_cost > type_.vision();        // cannot view in this terrain
+				double vision_red_to_green = 100.0 - 25.0 * m.vision_cost;
+
+				// passing true to select the less saturated red-to-green scale
+				std::string vision_color = game_config::red_to_green(vision_red_to_green, true).to_hex_string();
+				str << "<format>color='" << vision_color << "' text='";
 				// A 5 MP margin; if the vision costs go above
 				// the unit's vision + 5, we replace it with dashes.
 				if(cannot_view && (m.vision_cost > type_.vision() + 5)) {
-					str << font::unicode_figure_dash;
+					str_unformatted << font::unicode_figure_dash;
+				} else if(cannot_view) {
+					str_unformatted << "(" << m.vision_cost << ")";
 				} else {
-					str << m.vision_cost;
+					str_unformatted << m.vision_cost;
 				}
-				str << "'</format>";
-				markup = str.str();
-				str.str(clear_stringstream);
-				str << m.vision_cost;
-				row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
+				if(m.vision_cost != 0) {
+					const int vision_hexes_per_turn = type_.vision() / m.vision_cost;
+					str_unformatted << " ";
+					for(int i = 0; i < vision_hexes_per_turn; ++i) {
+						// Unicode horizontal black hexagon and Unicode zero width space (to allow a line break)
+						str_unformatted << "\u2b23\u200b";
+					}
+				}
+				str << str_unformatted.str() << "'</format>";
+				row.emplace_back(str.str(), font::pango_line_width(str_unformatted.str(), normal_font_size));
 			}
 
 			//jamming
 			if (has_jamming) {
 				str.str(clear_stringstream);
-				const bool cannot_jam = m.jamming_cost > type_.jamming();
-				if (cannot_jam) {		// cannot jamm in this terrain
-					color = "red";
-				} else if (m.jamming_cost > m.vision_cost) {
-					color = "yellow";
-				} else if (m.jamming_cost == m.vision_cost) {
-					color = "white";
-				} else {
-					color = "green";
-				}
-				str << "<format>color=" << color << " text='";
+				str_unformatted.str(clear_stringstream);
+				const bool cannot_jam = m.jamming_cost > type_.jamming();       // cannot jam in this terrain
+				double jamming_red_to_green = 100.0 - 25.0 * m.jamming_cost;
+
+				// passing true to select the less saturated red-to-green scale
+				std::string jamming_color = game_config::red_to_green(jamming_red_to_green, true).to_hex_string();
+				str << "<format>color='" << jamming_color << "' text='";
 				// A 5 MP margin; if the jamming costs go above
 				// the unit's jamming + 5, we replace it with dashes.
 				if (cannot_jam && m.jamming_cost > type_.jamming() + 5) {
-					str << font::unicode_figure_dash;
+					str_unformatted << font::unicode_figure_dash;
+				} else if(cannot_jam) {
+					str_unformatted << "(" << m.jamming_cost << ")";
 				} else {
-					str << m.jamming_cost;
+					str_unformatted << m.jamming_cost;
 				}
-				str << "'</format>";
-
-				push_tab_pair(row, str.str());
+				if(m.jamming_cost != 0) {
+					const int jamming_hexes_per_turn = type_.jamming() / m.jamming_cost;
+					str_unformatted << " ";
+					for(int i = 0; i < jamming_hexes_per_turn; ++i) {
+						// Unicode horizontal black hexagon and Unicode zero width space (to allow a line break)
+						str_unformatted << "\u2b23\u200b";
+					}
+				}
+				str << str_unformatted.str() << "'</format>";
+				row.emplace_back(str.str(), font::pango_line_width(str_unformatted.str(), normal_font_size));
 			}
 
 			table.push_back(row);
@@ -911,7 +921,7 @@ std::string unit_topic_generator::operator()() const {
 
 		ss << generate_table(table);
 	} else {
-		WRN_HP << "When building unit help topics, the display object was null and we couldn't get the terrain info we need.\n";
+		WRN_HP << "When building unit help topics, the display object was null and we couldn't get the terrain info we need.";
 	}
 	return ss.str();
 }
