@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2021
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -115,7 +115,7 @@ gamemap::gamemap(const std::string& data):
 		tdata_ = std::make_shared<terrain_type_data>(game_config_view::wrap({}));
 	}
 
-	DBG_G << "loading map: '" << data << "'\n";
+	DBG_G << "loading map: '" << data << "'";
 	read(data);
 }
 
@@ -232,10 +232,14 @@ void gamemap_base::overlay(const gamemap_base& m, map_location loc, const std::v
 	for(int x1 = xstart; x1 != xend; ++x1) {
 		int ystart, yend, yoffset;
 		if(x1 & 1) {
-			ystart = ystart_odd , yend = yend_odd , yoffset = yoffset_odd;
+			ystart = ystart_odd;
+			yend = yend_odd;
+			yoffset = yoffset_odd;
 		}
 		else {
-			ystart = ystart_even, yend = yend_even, yoffset = yoffset_even;
+			ystart = ystart_even;
+			yend = yend_even;
+			yoffset = yoffset_even;
 		}
 		for(int y1 = ystart; y1 != yend; ++y1) {
 			const int x2 = x1 + xoffset;
@@ -392,7 +396,7 @@ bool gamemap_base::on_board_with_border(const map_location& loc) const
 
 void gamemap::set_terrain(const map_location& loc, const t_translation::terrain_code & terrain, const terrain_type_data::merge_mode mode, bool replace_if_failed) {
 	if(!on_board_with_border(loc)) {
-		DBG_G << "set_terrain: " << loc << " is not on the map.\n";
+		DBG_G << "set_terrain: " << loc << " is not on the map.";
 		// off the map: ignore request
 		return;
 	}
