@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2006 - 2022
+	Copyright (C) 2006 - 2024
 	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
 	Copyright (C) 2003 by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
@@ -22,6 +22,7 @@
 #include "preferences/general.hpp"
 #include "sdl/rect.hpp"
 #include "tooltips.hpp"
+#include "sdl/input.hpp" // get_mouse_state
 
 static lg::log_domain log_display("display");
 #define WRN_DP LOG_STREAM(warn, log_display)
@@ -92,7 +93,7 @@ void mouse_handler_base::touch_motion_event(const SDL_TouchFingerEvent& event, c
 void mouse_handler_base::mouse_update(const bool browse, map_location loc)
 {
 	int x, y;
-	SDL_GetMouseState(&x, &y);
+	sdl::get_mouse_state(&x, &y);
 	mouse_motion(x, y, browse, true, loc);
 }
 
@@ -108,7 +109,7 @@ bool mouse_handler_base::mouse_motion_default(int x, int y, bool /*update*/)
 		// if the game is run in a window, we could miss a LMB/MMB up event
 		// if it occurs outside our window.
 		// thus, we need to check if the LMB/MMB is still down
-		minimap_scrolling_ = ((SDL_GetMouseState(nullptr, nullptr) & (SDL_BUTTON(SDL_BUTTON_LEFT) | SDL_BUTTON(SDL_BUTTON_MIDDLE))) != 0);
+		minimap_scrolling_ = ((sdl::get_mouse_button_mask() & (SDL_BUTTON(SDL_BUTTON_LEFT) | SDL_BUTTON(SDL_BUTTON_MIDDLE))) != 0);
 		if(minimap_scrolling_) {
 			const map_location& loc = gui().minimap_location_on(x, y);
 			if(loc.valid()) {
@@ -133,7 +134,7 @@ bool mouse_handler_base::mouse_motion_default(int x, int y, bool /*update*/)
 	int my = drag_from_y_;
 
 	if(is_dragging() && !dragging_started_) {
-		Uint32 mouse_state = dragging_left_ || dragging_right_ ? SDL_GetMouseState(&mx, &my) : 0;
+		Uint32 mouse_state = dragging_left_ || dragging_right_ ? sdl::get_mouse_state(&mx, &my) : 0;
 #ifdef MOUSE_TOUCH_EMULATION
 		if(dragging_left_ && (mouse_state & SDL_BUTTON(SDL_BUTTON_RIGHT))) {
 			// Monkey-patch touch controls again to make them look like left button.
@@ -157,6 +158,17 @@ bool mouse_handler_base::mouse_motion_default(int x, int y, bool /*update*/)
 	return false;
 }
 
+bool mouse_handler_base::mouse_button_event(const SDL_MouseButtonEvent& event, uint8_t button,
+											map_location loc, bool click)
+{
+	(void)event;
+	(void)button;
+	(void)loc;
+	(void)click;
+
+	return false;
+}
+
 void mouse_handler_base::mouse_press(const SDL_MouseButtonEvent& event, const bool browse)
 {
 	if(is_middle_click(event) && !preferences::middle_click_scrolls()) {
@@ -174,44 +186,60 @@ void mouse_handler_base::mouse_press(const SDL_MouseButtonEvent& event, const bo
 			cancel_dragging();
 			touch_timestamp = clock();
 			init_dragging(dragging_touch_);
-			left_click(event.x, event.y, browse);
+			if (!mouse_button_event(event, SDL_BUTTON_LEFT, loc, true)) {
+				left_click(event.x, event.y, browse);
+			}
 		} else if (event.state == SDL_RELEASED) {
 			minimap_scrolling_ = false;
 
 			if (!dragging_started_ && touch_timestamp > 0) {
-				time_t dt = clock() - touch_timestamp;
-				if (dt >= 0 && clock_t(dt) > CLOCKS_PER_SEC * 3 / 10) {
-					right_click(event.x, event.y, browse); // show_menu_ = true;
+				clock_t dt = clock() - touch_timestamp;
+				if (dt > CLOCKS_PER_SEC * 3 / 10) {
+					if (!mouse_button_event(event, SDL_BUTTON_RIGHT, loc, true)) {
+						// BUG: This function won't do anything in the game, need right_mouse_up()
+						right_click(event.x, event.y, browse); // show_menu_ = true;
+					}
 				}
 			} else {
 				touch_timestamp = 0;
 			}
 
 			clear_dragging(event, browse);
+			mouse_button_event(event, SDL_BUTTON_LEFT, loc);
 			left_mouse_up(event.x, event.y, browse);
+			clear_drag_from_hex();
 		}
 	} else if(is_left_click(event)) {
 		if(event.state == SDL_PRESSED) {
 			cancel_dragging();
 			init_dragging(dragging_left_);
-			left_click(event.x, event.y, browse);
+			if (!mouse_button_event(event, SDL_BUTTON_LEFT, loc, true)) {
+				left_click(event.x, event.y, browse);
+			}
 		} else if(event.state == SDL_RELEASED) {
 			minimap_scrolling_ = false;
 			clear_dragging(event, browse);
+			mouse_button_event(event, SDL_BUTTON_LEFT, loc);
 			left_mouse_up(event.x, event.y, browse);
+			clear_drag_from_hex();
 		}
 	} else if(is_right_click(event)) {
 		if(event.state == SDL_PRESSED) {
+			mouse_button_event(event, SDL_BUTTON_RIGHT, loc);
 			cancel_dragging();
 			init_dragging(dragging_right_);
 			right_click(event.x, event.y, browse);
 		} else if(event.state == SDL_RELEASED) {
 			minimap_scrolling_ = false;
 			clear_dragging(event, browse);
-			right_mouse_up(event.x, event.y, browse);
+			if (!mouse_button_event(event, SDL_BUTTON_RIGHT, loc, true)) {
+				right_mouse_up(event.x, event.y, browse);
+			}
+			clear_drag_from_hex();
 		}
 	} else if(is_middle_click(event)) {
 		if(event.state == SDL_PRESSED) {
+			drag_from_hex_ = loc;
 			set_scroll_start(event.x, event.y);
 			scroll_started_ = true;
 
@@ -222,17 +250,35 @@ void mouse_handler_base::mouse_press(const SDL_MouseButtonEvent& event, const bo
 				minimap_scrolling_ = true;
 				last_hex_ = minimap_loc;
 				gui().scroll_to_tile(minimap_loc, display::WARP, false);
+			} else if(mouse_button_event(event, SDL_BUTTON_MIDDLE, loc, true)) {
+				scroll_started_ = false;
+				simple_warp_ = false;
 			} else if(simple_warp_) {
 				// middle click not on minimap, check gamemap instead
 				if(loc.valid()) {
 					last_hex_ = loc;
 					gui().scroll_to_tile(loc, display::WARP, false);
 				}
+			} else {
+				// Deselect the current tile as we're scrolling
+				gui().highlight_hex({-1,-1});
 			}
 		} else if(event.state == SDL_RELEASED) {
 			minimap_scrolling_ = false;
 			simple_warp_ = false;
 			scroll_started_ = false;
+			mouse_button_event(event, SDL_BUTTON_MIDDLE, loc);
+			clear_drag_from_hex();
+		}
+	} else if(event.button == SDL_BUTTON_X1 || event.button == SDL_BUTTON_X2) {
+		if(event.state == SDL_PRESSED) {
+			cancel_dragging();
+			// record mouse-down hex in drag_from_hex_
+			drag_from_hex_ = loc;
+			mouse_button_event(event, event.button, loc);
+		} else {
+			mouse_button_event(event, event.button, loc, true);
+			clear_drag_from_hex();
 		}
 	}
 	if(!dragging_left_ && !dragging_right_ && !dragging_touch_ && dragging_started_) {
@@ -267,7 +313,7 @@ bool mouse_handler_base::is_right_click(const SDL_MouseButtonEvent& event) const
 	(void) event;
 	return false;
 #else
-    if(event.which == SDL_TOUCH_MOUSEID) {
+	if(event.which == SDL_TOUCH_MOUSEID) {
 		return false;
 	}
 	return event.button == SDL_BUTTON_RIGHT
@@ -311,13 +357,13 @@ void mouse_handler_base::left_drag_end(int /*x*/, int /*y*/, const bool browse)
 void mouse_handler_base::mouse_wheel(int scrollx, int scrolly, bool browse)
 {
 	int x, y;
-	SDL_GetMouseState(&x, &y);
+	sdl::get_mouse_state(&x, &y);
 
 	int movex = scrollx * preferences::scroll_speed();
 	int movey = scrolly * preferences::scroll_speed();
 
 	// Don't scroll map if cursor is not in gamemap area
-	if(!sdl::point_in_rect(x, y, gui().map_area())) {
+	if(!gui().map_area().contains(x, y)) {
 		return;
 	}
 
@@ -354,14 +400,14 @@ void mouse_handler_base::right_mouse_up(int x, int y, const bool browse)
 	if(m != nullptr) {
 		show_menu_ = true;
 	} else {
-		WRN_DP << "no context menu found..." << std::endl;
+		WRN_DP << "no context menu found...";
 	}
 }
 
 void mouse_handler_base::init_dragging(bool& dragging_flag)
 {
 	dragging_flag = true;
-	SDL_GetMouseState(&drag_from_x_, &drag_from_y_);
+	sdl::get_mouse_state(&drag_from_x_, &drag_from_y_);
 	drag_from_hex_ = gui().hex_clicked_on(drag_from_x_, drag_from_y_);
 }
 
@@ -406,4 +452,9 @@ void mouse_handler_base::clear_dragging(const SDL_MouseButtonEvent& event, bool
 	}
 }
 
+void mouse_handler_base::clear_drag_from_hex()
+{
+	drag_from_hex_ = map_location::null_location();
+}
+
 } // end namespace events
