@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2014 - 2022
+	Copyright (C) 2014 - 2024
 	by Chris Beck <render787@gmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -23,7 +23,7 @@
 #include<string>
 #include<vector>
 
-namespace mp_ui_alerts {
+namespace mp::ui_alerts {
 
 	// Functions called when such an event occurs
 	void player_joins(bool is_lobby);
