@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2005 - 2022
+	Copyright (C) 2005 - 2024
 	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
 	Copyright (C) 2003 by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
@@ -147,10 +147,10 @@ plain_route a_star_search(const map_location& src, const map_location& dst,
 	assert(stop_at <= calc.getNoPathValue());
 	//---------------------------------------------------
 
-	DBG_PF << "A* search: " << src << " -> " << dst << '\n';
+	DBG_PF << "A* search: " << src << " -> " << dst;
 
 	if (calc.cost(dst, 0) >= stop_at) {
-		LOG_PF << "aborted A* search because Start or Dest is invalid\n";
+		LOG_PF << "aborted A* search because Start or Dest is invalid";
 		plain_route locRoute;
 		locRoute.move_cost = static_cast<int>(calc.getNoPathValue());
 		return locRoute;
@@ -219,7 +219,7 @@ plain_route a_star_search(const map_location& src, const map_location& dst,
 
 	plain_route route;
 	if (nodes[index(dst)].g <= stop_at) {
-		DBG_PF << "found solution; calculating it...\n";
+		DBG_PF << "found solution; calculating it...";
 		route.move_cost = static_cast<int>(nodes[index(dst)].g);
 		for (node curr = nodes[index(dst)]; curr.prev != map_location::null_location(); curr = nodes[index(curr.prev)]) {
 			route.steps.push_back(curr.curr);
@@ -227,7 +227,7 @@ plain_route a_star_search(const map_location& src, const map_location& dst,
 		route.steps.push_back(src);
 		std::reverse(route.steps.begin(), route.steps.end());
 	} else {
-		LOG_PF << "aborted a* search  " << "\n";
+		LOG_PF << "aborted a* search  ";
 		route.move_cost = static_cast<int>(calc.getNoPathValue());
 	}
 
