@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2005 - 2022
+	Copyright (C) 2005 - 2024
 	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
 	Copyright (C) 2003 by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
@@ -33,7 +33,6 @@
 #include "units/map.hpp"
 #include "wml_exception.hpp"
 
-#include <iostream>
 #include <vector>
 #include <algorithm>
 
@@ -416,7 +415,7 @@ static void find_routes(
 				}
 			}
 
-			if ( viewing_team && current_team && viewing_team != current_team && viewing_team->shrouded(next_hex) ) {
+			if ( !see_all && viewing_team && current_team && viewing_team != current_team && viewing_team->shrouded(next_hex) ) {
 				// bug #2199: in "Show Enemy Moves", don't pathfind enemy units through the player's shroud
 				continue;
 			}
