@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -30,6 +30,7 @@
 #include "serialization/base64.hpp"
 #include "serialization/string_utils.hpp"
 #include "sdl/rect.hpp"
+#include "sdl/texture.hpp"
 
 #include <SDL2/SDL_image.h>
 
@@ -38,78 +39,68 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/functional/hash_fwd.hpp>
 
+#include <array>
 #include <set>
 
-static lg::log_domain log_display("display");
-#define ERR_DP LOG_STREAM(err, log_display)
-#define LOG_DP LOG_STREAM(info, log_display)
+static lg::log_domain log_image("image");
+#define ERR_IMG LOG_STREAM(err, log_image)
+#define WRN_IMG LOG_STREAM(warn, log_image)
+#define LOG_IMG LOG_STREAM(info, log_image)
+#define DBG_IMG LOG_STREAM(debug, log_image)
 
 static lg::log_domain log_config("config");
 #define ERR_CFG LOG_STREAM(err, log_config)
 
 using game_config::tile_size;
 
-template<typename T>
-struct cache_item
-{
-	cache_item()
-		: item()
-		, loaded(false)
-	{
-	}
-
-	cache_item(const T& item)
-		: item(item)
-		, loaded(true)
-	{
-	}
-
-	T item;
-	bool loaded;
-};
-
-namespace std
-{
 template<>
-struct hash<image::locator::value>
+struct std::hash<image::locator::value>
 {
 	std::size_t operator()(const image::locator::value& val) const
 	{
-		std::size_t hash = std::hash<unsigned>{}(val.type_);
+		std::size_t hash = std::hash<unsigned>{}(val.type);
 
-		if(val.type_ == image::locator::FILE || val.type_ == image::locator::SUB_FILE) {
-			boost::hash_combine(hash, val.filename_);
+		if(val.type == image::locator::FILE || val.type == image::locator::SUB_FILE) {
+			boost::hash_combine(hash, val.filename);
 		}
 
-		if(val.type_ == image::locator::SUB_FILE) {
-			boost::hash_combine(hash, val.loc_.x);
-			boost::hash_combine(hash, val.loc_.y);
-			boost::hash_combine(hash, val.center_x_);
-			boost::hash_combine(hash, val.center_y_);
-			boost::hash_combine(hash, val.modifications_);
+		if(val.type == image::locator::SUB_FILE) {
+			boost::hash_combine(hash, val.loc.x);
+			boost::hash_combine(hash, val.loc.y);
+			boost::hash_combine(hash, val.center_x);
+			boost::hash_combine(hash, val.center_y);
+			boost::hash_combine(hash, val.modifications);
 		}
 
 		return hash;
 	}
 };
-}
 
 namespace image
 {
 template<typename T>
 class cache_type
 {
-public:
-	cache_type()
-		: content_()
-	{
-	}
+	using Key = locator::value;
 
-	cache_item<T>& get_element(int index)
+public:
+	struct cache_item
 	{
-		if(static_cast<unsigned>(index) >= content_.size())
-			content_.resize(index + 1);
-		return content_[index];
+		T item {};
+		bool loaded = false;
+
+		void populate(T&& value)
+		{
+			item = value;
+			loaded = true;
+		}
+	};
+
+	cache_type() = default;
+
+	cache_item& get_element(const Key& item)
+	{
+		return content_[item];
 	}
 
 	void flush()
@@ -118,45 +109,59 @@ public:
 	}
 
 private:
-	std::vector<cache_item<T>> content_;
+	std::unordered_map<Key, cache_item> content_;
 };
 
+std::size_t locator::hash() const
+{
+	return std::hash<value>{}(val_);
+}
+
 template<typename T>
 bool locator::in_cache(cache_type<T>& cache) const
 {
-	return index_ < 0 ? false : cache.get_element(index_).loaded;
+	return cache.get_element(val_).loaded;
 }
 
 template<typename T>
 const T& locator::locate_in_cache(cache_type<T>& cache) const
 {
-	static T dummy;
-	return index_ < 0 ? dummy : cache.get_element(index_).item;
+	return cache.get_element(val_).item;
 }
 
 template<typename T>
 T& locator::access_in_cache(cache_type<T>& cache) const
 {
-	static T dummy;
-	return index_ < 0 ? dummy : cache.get_element(index_).item;
+	return cache.get_element(val_).item;
 }
 
 template<typename T>
-void locator::add_to_cache(cache_type<T>& cache, const T& data) const
+std::optional<T> locator::copy_from_cache(cache_type<T>& cache) const
 {
-	if(index_ >= 0) {
-		cache.get_element(index_) = cache_item<T>(data);
-	}
+	const auto& elem = cache.get_element(val_);
+	return elem.loaded ? std::make_optional(elem.item) : std::nullopt;
 }
+
+template<typename T>
+void locator::add_to_cache(cache_type<T>& cache, T data) const
+{
+	cache.get_element(val_).populate(std::move(data));
 }
 
 namespace
 {
-image::locator::locator_finder_t locator_finder;
-
 /** Definition of all image maps */
-image::image_cache images_, scaled_to_zoom_, hexed_images_, scaled_to_hex_images_, tod_colored_images_,
-		brightened_images_;
+std::array<surface_cache, NUM_TYPES> surfaces_;
+
+/**
+ * Texture caches.
+ * Note that the latter two are temporary and should be removed once we have OGL and shader support.
+ */
+using texture_cache_map = std::map<image::scale_quality, image::texture_cache>;
+
+texture_cache_map textures_;
+texture_cache_map textures_hexed_;
+texture_cache_map texture_tod_colored_;
 
 // cache storing if each image fit in a hex
 image::bool_cache in_hex_info_;
@@ -165,9 +170,16 @@ image::bool_cache in_hex_info_;
 image::bool_cache is_empty_hex_;
 
 // caches storing the different lighted cases for each image
-image::lit_cache lit_images_, lit_scaled_images_;
+image::lit_surface_cache lit_surfaces_;
+image::lit_texture_cache lit_textures_;
 // caches storing each lightmap generated
-image::lit_variants lightmaps_;
+image::lit_surface_variants surface_lightmaps_;
+image::lit_texture_variants texture_lightmaps_;
+
+// diagnostics for tracking skipped cache impact
+std::array<bool_cache, NUM_TYPES> skipped_cache_;
+int duplicate_loads_ = 0;
+int total_loads_ = 0;
 
 // const int cache_version_ = 0;
 
@@ -176,13 +188,8 @@ std::map<std::string, bool> image_existence_map;
 // directories where we already cached file existence
 std::set<std::string> precached_dirs;
 
-std::map<surface, surface> reversed_images_;
-
 int red_adjust = 0, green_adjust = 0, blue_adjust = 0;
 
-unsigned int zoom = tile_size;
-unsigned int cached_zoom = 0;
-
 const std::string data_uri_prefix = "data:";
 struct parsed_data_URI{
 	explicit parsed_data_URI(std::string_view data_URI);
@@ -211,209 +218,112 @@ parsed_data_URI::parsed_data_URI(std::string_view data_URI)
 
 } // end anon namespace
 
-namespace image
-{
 mini_terrain_cache_map mini_terrain_cache;
 mini_terrain_cache_map mini_fogged_terrain_cache;
 mini_terrain_cache_map mini_highlighted_terrain_cache;
 
-static int last_index_ = 0;
-
 void flush_cache()
 {
-	{
-		images_.flush();
-		hexed_images_.flush();
-		tod_colored_images_.flush();
-		scaled_to_zoom_.flush();
-		scaled_to_hex_images_.flush();
-		brightened_images_.flush();
-		lit_images_.flush();
-		lit_scaled_images_.flush();
-		in_hex_info_.flush();
-		is_empty_hex_.flush();
-		mini_terrain_cache.clear();
-		mini_fogged_terrain_cache.clear();
-		mini_highlighted_terrain_cache.clear();
-		reversed_images_.clear();
-		image_existence_map.clear();
-		precached_dirs.clear();
+	for(surface_cache& cache : surfaces_) {
+		cache.flush();
 	}
-	/* We can't reset last_index_, since some locators are still alive
-	   when using :refresh. That would cause them to point to the wrong
-	   images. Not resetting the variable causes a memory leak, though. */
-	// last_index_ = 0;
+	lit_surfaces_.flush();
+	lit_textures_.flush();
+	surface_lightmaps_.clear();
+	texture_lightmaps_.clear();
+	in_hex_info_.flush();
+	is_empty_hex_.flush();
+	textures_.clear();
+	textures_hexed_.clear();
+	texture_tod_colored_.clear();
+	mini_terrain_cache.clear();
+	mini_fogged_terrain_cache.clear();
+	mini_highlighted_terrain_cache.clear();
+	image_existence_map.clear();
+	precached_dirs.clear();
 }
 
-void locator::init_index()
-{
-	auto i = locator_finder.find(val_);
 
-	if(i == locator_finder.end()) {
-		index_ = last_index_++;
-		locator_finder.emplace(val_, index_);
-	} else {
-		index_ = i->second;
+
+locator locator::clone(const std::string& mods) const
+{
+	locator res = *this;
+	if(!mods.empty()) {
+		res.val_.modifications += mods;
+		res.val_.type = SUB_FILE;
 	}
+
+	return res;
 }
 
-void locator::parse_arguments()
+std::ostream& operator<<(std::ostream& s, const locator& l)
 {
-	std::string& fn = val_.filename_;
-	if(fn.empty()) {
+	s << l.get_filename();
+	if(!l.get_modifications().empty()) {
+		if(l.get_modifications()[0] != '~') {
+			s << '~';
+		}
+		s << l.get_modifications();
+	}
+	return s;
+}
+
+locator::value::value(const std::string& fn)
+	: type(FILE)
+	, filename(fn)
+{
+	if(filename.empty()) {
 		return;
 	}
 
-	if(boost::algorithm::starts_with(fn, data_uri_prefix)) {
-		parsed_data_URI parsed{fn};
-
-		if(!parsed.good) {
-			std::string_view view{ fn };
+	if(boost::algorithm::starts_with(filename, data_uri_prefix)) {
+		if(parsed_data_URI parsed{ filename }; !parsed.good) {
+			std::string_view view{ filename };
 			std::string_view stripped = view.substr(0, view.find(","));
-			ERR_DP << "Invalid data URI: " << stripped << std::endl;
+			ERR_IMG << "Invalid data URI: " << stripped;
 		}
 
-		val_.is_data_uri_ = true;
+		is_data_uri = true;
 	}
 
-	std::size_t markup_field = fn.find('~');
-
-	if(markup_field != std::string::npos) {
-		val_.type_ = SUB_FILE;
-		val_.modifications_ = fn.substr(markup_field, fn.size() - markup_field);
-		fn = fn.substr(0, markup_field);
+	if(const std::size_t markup_field = filename.find('~'); markup_field != std::string::npos) {
+		type = SUB_FILE;
+		modifications = filename.substr(markup_field, filename.size() - markup_field);
+		filename = filename.substr(0, markup_field);
 	}
 }
 
-locator::locator()
-	: index_(-1)
-	, val_()
-{
-}
-
-locator::locator(const locator& a, const std::string& mods)
-	: index_(-1)
-	, val_(a.val_)
-{
-	if(!mods.empty()) {
-		val_.modifications_ += mods;
-		val_.type_ = SUB_FILE;
-		init_index();
-	} else {
-		index_ = a.index_;
-	}
-}
-
-locator::locator(const char* filename)
-	: index_(-1)
-	, val_(filename)
-{
-	parse_arguments();
-	init_index();
-}
-
-locator::locator(const std::string& filename)
-	: index_(-1)
-	, val_(filename)
-{
-	parse_arguments();
-	init_index();
-}
-
-locator::locator(const std::string& filename, const std::string& modifications)
-	: index_(-1)
-	, val_(filename, modifications)
-{
-	init_index();
-}
-
-locator::locator(const std::string& filename,
-		const map_location& loc,
-		int center_x,
-		int center_y,
-		const std::string& modifications)
-	: index_(-1)
-	, val_(filename, loc, center_x, center_y, modifications)
-{
-	init_index();
-}
-
-locator& locator::operator=(const locator& a)
-{
-	index_ = a.index_;
-	val_ = a.val_;
-
-	return *this;
-}
-
-locator::value::value()
-	: type_(NONE)
-	, is_data_uri_(false)
-	, filename_()
-	, loc_()
-	, modifications_()
-	, center_x_(0)
-	, center_y_(0)
-{
-}
-
-locator::value::value(const char* filename)
-	: type_(FILE)
-	, is_data_uri_(false)
-	, filename_(filename)
-	, loc_()
-	, modifications_()
-	, center_x_(0)
-	, center_y_(0)
-{
-}
-
-locator::value::value(const std::string& filename)
-	: type_(FILE)
-	, is_data_uri_(false)
-	, filename_(filename)
-	, loc_()
-	, modifications_()
-	, center_x_(0)
-	, center_y_(0)
-{
-}
-
 locator::value::value(const std::string& filename, const std::string& modifications)
-	: type_(SUB_FILE)
-	, is_data_uri_(false)
-	, filename_(filename)
-	, loc_()
-	, modifications_(modifications)
-	, center_x_(0)
-	, center_y_(0)
+	: type(SUB_FILE)
+	, filename(filename)
+	, modifications(modifications)
 {
 }
 
-locator::value::value(const std::string& filename,
+locator::value::value(
+		const std::string& filename,
 		const map_location& loc,
 		int center_x,
 		int center_y,
 		const std::string& modifications)
-	: type_(SUB_FILE)
-	, is_data_uri_(false)
-	, filename_(filename)
-	, loc_(loc)
-	, modifications_(modifications)
-	, center_x_(center_x)
-	, center_y_(center_y)
+	: type(SUB_FILE)
+	, filename(filename)
+	, modifications(modifications)
+	, loc(loc)
+	, center_x(center_x)
+	, center_y(center_y)
 {
 }
 
 bool locator::value::operator==(const value& a) const
 {
-	if(a.type_ != type_) {
+	if(a.type != type) {
 		return false;
-	} else if(type_ == FILE) {
-		return filename_ == a.filename_;
-	} else if(type_ == SUB_FILE) {
-		return filename_ == a.filename_ && loc_ == a.loc_ && modifications_ == a.modifications_
-			   && center_x_ == a.center_x_ && center_y_ == a.center_y_;
+	} else if(type == FILE) {
+		return filename == a.filename;
+	} else if(type == SUB_FILE) {
+		return std::tie(filename, loc, modifications, center_x, center_y) ==
+			std::tie(a.filename, a.loc, a.modifications, a.center_x, a.center_y);
 	}
 
 	return false;
@@ -421,20 +331,13 @@ bool locator::value::operator==(const value& a) const
 
 bool locator::value::operator<(const value& a) const
 {
-	if(type_ != a.type_) {
-		return type_ < a.type_;
-	} else if(type_ == FILE) {
-		return filename_ < a.filename_;
-	} else if(type_ == SUB_FILE) {
-		if(filename_ != a.filename_)
-			return filename_ < a.filename_;
-		if(loc_ != a.loc_)
-			return loc_ < a.loc_;
-		if(center_x_ != a.center_x_)
-			return center_x_ < a.center_x_;
-		if(center_y_ != a.center_y_)
-			return center_y_ < a.center_y_;
-		return (modifications_ < a.modifications_);
+	if(type != a.type) {
+		return type < a.type;
+	} else if(type == FILE) {
+		return filename < a.filename;
+	} else if(type == SUB_FILE) {
+		return std::tie(filename, loc, modifications, center_x, center_y) <
+			std::tie(a.filename, a.loc, a.modifications, a.center_x, a.center_y);
 	}
 
 	return false;
@@ -457,8 +360,22 @@ static void add_localized_overlay(const std::string& ovr_file, surface& orig_sur
 static surface load_image_file(const image::locator& loc)
 {
 	surface res;
+	const std::string& name = loc.get_filename();
 
-	std::string location = filesystem::get_binary_file_location("images", loc.get_filename());
+	std::string location = filesystem::get_binary_file_location("images", name);
+
+	// Many images have been converted from PNG to WEBP format,
+	// but the old filename may still be saved in savegame files etc.
+	// If the file does not exist in ".png" format, also try ".webp".
+	// Similarly for ".jpg", which conveniently has the same number of letters as ".png".
+	if(location.empty() && (filesystem::ends_with(name, ".png") || filesystem::ends_with(name, ".jpg"))) {
+		std::string webp_name = name.substr(0, name.size() - 4) + ".webp";
+		location = filesystem::get_binary_file_location("images", webp_name);
+		if(!location.empty()) {
+			WRN_IMG << "Replaced missing '" << name << "' with found '"
+			        << webp_name << "'.";
+		}
+	}
 
 	{
 		if(!location.empty()) {
@@ -481,10 +398,12 @@ static surface load_image_file(const image::locator& loc)
 		}
 	}
 
-	if(!res && !loc.get_filename().empty()) {
-		ERR_DP << "could not open image '" << loc.get_filename() << "'" << std::endl;
-		if(game_config::debug && loc.get_filename() != game_config::images::missing)
-			return get_image(game_config::images::missing, UNSCALED);
+	if(!res && !name.empty()) {
+		ERR_IMG << "could not open image '" << name << "'";
+		if(game_config::debug && name != game_config::images::missing)
+			return get_surface(game_config::images::missing, UNSCALED);
+		if(name != game_config::images::blank)
+			return get_surface(game_config::images::blank, UNSCALED);
 	}
 
 	return res;
@@ -492,7 +411,7 @@ static surface load_image_file(const image::locator& loc)
 
 static surface load_image_sub_file(const image::locator& loc)
 {
-	surface surf = get_image(loc.get_filename(), UNSCALED);
+	surface surf = get_surface(loc.get_filename(), UNSCALED);
 	if(surf == nullptr) {
 		return nullptr;
 	}
@@ -515,7 +434,7 @@ static surface load_image_sub_file(const image::locator& loc)
 			ERR_CFG << "Failed to apply a modification to an image:\n"
 					<< "Image: " << loc.get_filename() << "\n"
 					<< "Modifications: " << ss.str() << "\n"
-					<< "Error: " << e.message << "\n";
+					<< "Error: " << e.message;
 		}
 
 		// NOTE: do this *after* applying the mod or you'll get crashes!
@@ -523,7 +442,7 @@ static surface load_image_sub_file(const image::locator& loc)
 	}
 
 	if(loc.get_loc().valid()) {
-		SDL_Rect srcrect = sdl::create_rect(
+		rect srcrect(
 			((tile_size * 3) / 4)                           *  loc.get_loc().x,
 			  tile_size * loc.get_loc().y + (tile_size / 2) * (loc.get_loc().x % 2),
 			  tile_size,
@@ -563,21 +482,21 @@ static surface load_image_data_uri(const image::locator& loc)
 	if(!parsed.good) {
 		std::string_view fn = loc.get_filename();
 		std::string_view stripped = fn.substr(0, fn.find(","));
-		ERR_DP << "Invalid data URI: " << stripped << std::endl;
+		ERR_IMG << "Invalid data URI: " << stripped;
 	} else if(parsed.mime.substr(0, 5) != "image") {
-		ERR_DP << "Data URI not of image MIME type: " << parsed.mime << std::endl;
+		ERR_IMG << "Data URI not of image MIME type: " << parsed.mime;
 	} else {
 		const std::vector<uint8_t> image_data = base64::decode(parsed.data);
-		filesystem::rwops_ptr rwops{SDL_RWFromConstMem(image_data.data(), image_data.size()), &SDL_FreeRW};
+		filesystem::rwops_ptr rwops{SDL_RWFromConstMem(image_data.data(), image_data.size())};
 
 		if(image_data.empty()) {
-			ERR_DP << "Invalid encoding in data URI" << std::endl;
+			ERR_IMG << "Invalid encoding in data URI";
 		} else if(parsed.mime == "image/png") {
 			surf = IMG_LoadTyped_RW(rwops.release(), true, "PNG");
 		} else if(parsed.mime == "image/jpeg") {
 			surf = IMG_LoadTyped_RW(rwops.release(), true, "JPG");
 		} else {
-			ERR_DP << "Invalid image MIME type: " << parsed.mime << std::endl;
+			ERR_IMG << "Invalid image MIME type: " << parsed.mime;
 		}
 	}
 
@@ -614,8 +533,8 @@ static surface apply_light(surface surf, const light_string& ls)
 
 	// check if the lightmap is already cached or need to be generated
 	surface lightmap = nullptr;
-	auto i = lightmaps_.find(ls);
-	if(i != lightmaps_.end()) {
+	auto i = surface_lightmaps_.find(ls);
+	if(i != surface_lightmaps_.end()) {
 		lightmap = i->second;
 	} else {
 		// build all the paths for lightmap sources
@@ -637,7 +556,7 @@ static surface apply_light(surface surf, const light_string& ls)
 			// get the corresponding image and apply the lightmap operation to it
 			// This allows to also cache lightmap parts.
 			// note that we avoid infinite recursion by using only atomic operation
-			surface lts = image::get_lighted_image(lm_img[sls[0]], sls, HEXED);
+			surface lts = image::get_lighted_image(lm_img[sls[0]], sls);
 
 			// first image will be the base where we blit the others
 			if(lightmap == nullptr) {
@@ -649,7 +568,7 @@ static surface apply_light(surface surf, const light_string& ls)
 		}
 
 		// cache the result
-		lightmaps_[ls] = lightmap;
+		surface_lightmaps_[ls] = lightmap;
 	}
 
 	// apply the final lightmap
@@ -658,12 +577,12 @@ static surface apply_light(surface surf, const light_string& ls)
 
 bool locator::file_exists() const
 {
-	return val_.is_data_uri_
-		? parsed_data_URI{val_.filename_}.good
-		: !filesystem::get_binary_file_location("images", val_.filename_).empty();
+	return val_.is_data_uri
+		? parsed_data_URI{val_.filename}.good
+		: !filesystem::get_binary_file_location("images", val_.filename).empty();
 }
 
-surface load_from_disk(const locator& loc)
+static surface load_from_disk(const locator& loc)
 {
 	switch(loc.get_type()) {
 	case locator::FILE:
@@ -694,111 +613,63 @@ void set_color_adjustment(int r, int g, int b)
 		red_adjust = r;
 		green_adjust = g;
 		blue_adjust = b;
-		tod_colored_images_.flush();
-		brightened_images_.flush();
-		lit_images_.flush();
-		lit_scaled_images_.flush();
-		reversed_images_.clear();
+		surfaces_[TOD_COLORED].flush();
+		lit_surfaces_.flush();
+		lit_textures_.flush();
+		texture_tod_colored_.clear();
 	}
 }
 
-void set_zoom(unsigned int amount)
+static surface get_hexed(const locator& i_locator, bool skip_cache = false)
 {
-	if(amount != zoom) {
-		zoom = amount;
-		tod_colored_images_.flush();
-		brightened_images_.flush();
-		reversed_images_.clear();
-
-		// We keep these caches if:
-		// we use default zoom (it doesn't need those)
-		// or if they are already at the wanted zoom.
-		if(zoom != tile_size && zoom != cached_zoom) {
-			scaled_to_zoom_.flush();
-			scaled_to_hex_images_.flush();
-			lit_scaled_images_.flush();
-			cached_zoom = zoom;
+	surface image(get_surface(i_locator, UNSCALED, skip_cache));
+	surface mask(get_hexmask());
+	// Ensure the image is the correct size by cropping and/or centering.
+	// TODO: this should probably be a function of sdl/utils
+	if(image && (image->w != mask->w || image->h != mask->h)) {
+		DBG_IMG << "adjusting [" << image->w << ',' << image->h << ']'
+			<< " image to hex mask: " << i_locator;
+		// the fitted surface
+		surface fit(mask->w, mask->h);
+		// if the image is too large in either dimension, crop it.
+		if(image->w > mask->w || image->h >= mask->h) {
+			// fill the crop surface with transparency
+			sdl::fill_surface_rect(fit, nullptr,
+				SDL_MapRGBA(fit->format, 0, 0, 0, 0)
+			);
+			// crop the input image to hexmask dimensions
+			int cutx = std::max(0, image->w - mask->w) / 2;
+			int cuty = std::max(0, image->h - mask->h) / 2;
+			int cutw = std::min(image->w, mask->w);
+			int cuth = std::min(image->h, mask->h);
+			image = cut_surface(image, {cutx, cuty, cutw, cuth});
+			// image will now have dimensions <= mask
 		}
+		// center image
+		int placex = (mask->w - image->w) / 2;
+		int placey = (mask->h - image->h) / 2;
+		rect dst = {placex, placey, image->w, image->h};
+		sdl_blit(image, nullptr, fit, &dst);
+		image = fit;
 	}
-}
-
-static surface get_hexed(const locator& i_locator)
-{
-	surface image(get_image(i_locator, UNSCALED));
 	// hex cut tiles, also check and cache if empty result
 	bool is_empty = false;
-	surface res = mask_surface(image, get_hexmask(), &is_empty, i_locator.get_filename());
+	surface res = mask_surface(image, mask, &is_empty, i_locator.get_filename());
 	i_locator.add_to_cache(is_empty_hex_, is_empty);
 	return res;
 }
 
-static surface get_scaled_to_hex(const locator& i_locator)
+static surface get_tod_colored(const locator& i_locator, bool skip_cache = false)
 {
-	surface img = get_image(i_locator, HEXED);
-	// return scale_surface(img, zoom, zoom);
-
-	if(img) {
-		return scale_surface_nn(img, zoom, zoom);
-	}
-
-	return surface(nullptr);
-
-}
-
-static surface get_tod_colored(const locator& i_locator)
-{
-	surface img = get_image(i_locator, SCALED_TO_HEX);
+	surface img = get_surface(i_locator, HEXED, skip_cache);
 	return adjust_surface_color(img, red_adjust, green_adjust, blue_adjust);
 }
 
-static surface get_scaled_to_zoom(const locator& i_locator)
-{
-	assert(zoom != tile_size);
-	assert(tile_size != 0);
-
-	surface res(get_image(i_locator, UNSCALED));
-	// For some reason haloes seems to have invalid images, protect against crashing
-	if(res) {
-		return scale_surface_nn(res, ((res->w * zoom) / tile_size), ((res->h * zoom) / tile_size));
-	}
-
-	return surface(nullptr);
-}
-
-static surface get_brightened(const locator& i_locator)
-{
-	surface image(get_image(i_locator, TOD_COLORED));
-	return brighten_image(image, ftofxp(game_config::hex_brightening));
-}
-
 /** translate type to a simpler one when possible */
 static TYPE simplify_type(const image::locator& i_locator, TYPE type)
 {
-	switch(type) {
-	case SCALED_TO_ZOOM:
-		if(zoom == tile_size) {
-			type = UNSCALED;
-		}
-
-		break;
-	case BRIGHTENED:
-		if(ftofxp(game_config::hex_brightening) == ftofxp(1.0)) {
-			type = TOD_COLORED;
-		}
-
-		break;
-	default:
-		break;
-	}
-
 	if(type == TOD_COLORED) {
 		if(red_adjust == 0 && green_adjust == 0 && blue_adjust == 0) {
-			type = SCALED_TO_HEX;
-		}
-	}
-
-	if(type == SCALED_TO_HEX) {
-		if(zoom == tile_size) {
 			type = HEXED;
 		}
 	}
@@ -813,7 +684,10 @@ static TYPE simplify_type(const image::locator& i_locator, TYPE type)
 	return type;
 }
 
-surface get_image(const image::locator& i_locator, TYPE type)
+surface get_surface(
+	const image::locator& i_locator,
+	TYPE type,
+	bool skip_cache)
 {
 	surface res;
 
@@ -823,39 +697,20 @@ surface get_image(const image::locator& i_locator, TYPE type)
 
 	type = simplify_type(i_locator, type);
 
-	image_cache* imap;
 	// select associated cache
-	switch(type) {
-	case UNSCALED:
-		imap = &images_;
-		break;
-	case TOD_COLORED:
-		imap = &tod_colored_images_;
-		break;
-	case SCALED_TO_ZOOM:
-		imap = &scaled_to_zoom_;
-		break;
-	case HEXED:
-		imap = &hexed_images_;
-		break;
-	case SCALED_TO_HEX:
-		imap = &scaled_to_hex_images_;
-		break;
-	case BRIGHTENED:
-		imap = &brightened_images_;
-		break;
-	default:
+	if(type >= NUM_TYPES) {
+		WRN_IMG << "get_surface called with unknown image type";
 		return res;
 	}
+	surface_cache& imap = surfaces_[type];
 
 	// return the image if already cached
-	bool tmp = i_locator.in_cache(*imap);
-
-	if(tmp) {
-		surface result = i_locator.locate_in_cache(*imap);
-		return result;
+	if(auto cached_item = i_locator.copy_from_cache(imap)) {
+		return *cached_item;
 	}
 
+	DBG_IMG << "surface cache [" << type << "] miss: " << i_locator;
+
 	// not cached, generate it
 	switch(type) {
 	case UNSCALED:
@@ -863,74 +718,64 @@ surface get_image(const image::locator& i_locator, TYPE type)
 		res = load_from_disk(i_locator);
 		break;
 	case TOD_COLORED:
-		res = get_tod_colored(i_locator);
-		break;
-	case SCALED_TO_ZOOM:
-		res = get_scaled_to_zoom(i_locator);
+		res = get_tod_colored(i_locator, skip_cache);
 		break;
 	case HEXED:
-		res = get_hexed(i_locator);
-		break;
-	case SCALED_TO_HEX:
-		res = get_scaled_to_hex(i_locator);
-		break;
-	case BRIGHTENED:
-		res = get_brightened(i_locator);
+		res = get_hexed(i_locator, skip_cache);
 		break;
 	default:
-		return res;
+		throw game::error("get_surface somehow lost image type?");
 	}
 
-	i_locator.add_to_cache(*imap, res);
+	bool_cache& skip = skipped_cache_[type];
+	if(i_locator.in_cache(skip) && i_locator.locate_in_cache(skip))
+	{
+		DBG_IMG << "duplicate load: " << i_locator
+			<< " [" << type << "]"
+			<< " (" << duplicate_loads_ << "/" << total_loads_ << " total)";
+		++duplicate_loads_;
+	}
+	++total_loads_;
+
+	if(skip_cache) {
+		DBG_IMG << "surface cache [" << type << "] skip: " << i_locator;
+		i_locator.add_to_cache(skip, true);
+	} else {
+		i_locator.add_to_cache(imap, res);
+	}
 
 	return res;
 }
 
-surface get_lighted_image(const image::locator& i_locator, const light_string& ls, TYPE type)
+surface get_lighted_image(const image::locator& i_locator, const light_string& ls)
 {
 	surface res;
 	if(i_locator.is_void()) {
 		return res;
 	}
 
-	if(type == SCALED_TO_HEX && zoom == tile_size) {
-		type = HEXED;
-	}
-
 	// select associated cache
-	lit_cache* imap = &lit_images_;
-	if(type == SCALED_TO_HEX) {
-		imap = &lit_scaled_images_;
-	}
+	lit_surface_cache* imap = &lit_surfaces_;
 
 	// if no light variants yet, need to add an empty map
 	if(!i_locator.in_cache(*imap)) {
-		i_locator.add_to_cache(*imap, lit_variants());
+		i_locator.add_to_cache(*imap, lit_surface_variants());
 	}
 
 	// need access to add it if not found
 	{ // enclose reference pointing to data stored in a changing vector
-		const lit_variants& lvar = i_locator.locate_in_cache(*imap);
+		const lit_surface_variants& lvar = i_locator.locate_in_cache(*imap);
 		auto lvi = lvar.find(ls);
 		if(lvi != lvar.end()) {
 			return lvi->second;
 		}
 	}
 
+	DBG_IMG << "lit surface cache miss: " << i_locator;
+
 	// not cached yet, generate it
-	switch(type) {
-	case HEXED:
-		res = get_image(i_locator, HEXED);
-		res = apply_light(res, ls);
-		break;
-	case SCALED_TO_HEX:
-		// we light before scaling to reuse the unscaled cache
-		res = get_lighted_image(i_locator, ls, HEXED);
-		res = scale_surface_nn(res, zoom, zoom);
-		break;
-	default:
-		break;
-	}
+	res = get_surface(i_locator, HEXED);
+	res = apply_light(res, ls);
 
 	// record the lighted surface in the corresponding variants cache
 	i_locator.access_in_cache(*imap)[ls] = res;
@@ -938,10 +783,56 @@ surface get_lighted_image(const image::locator& i_locator, const light_string& l
 	return res;
 }
 
+texture get_lighted_texture(
+	const image::locator& i_locator,
+	const light_string& ls)
+{
+	if(i_locator.is_void()) {
+		return texture();
+	}
+
+	// select associated cache
+	lit_texture_cache* imap = &lit_textures_;
+
+	// if no light variants yet, need to add an empty map
+	if(!i_locator.in_cache(*imap)) {
+		i_locator.add_to_cache(*imap, lit_texture_variants());
+	}
+
+	// need access to add it if not found
+	{ // enclose reference pointing to data stored in a changing vector
+		const lit_texture_variants& lvar = i_locator.locate_in_cache(*imap);
+		auto lvi = lvar.find(ls);
+		if(lvi != lvar.end()) {
+			return lvi->second;
+		}
+	}
+
+	DBG_IMG << "lit texture cache miss: " << i_locator;
+
+	// not cached yet, generate it
+	texture tex(get_lighted_image(i_locator, ls));
+
+	// record the lighted texture in the corresponding variants cache
+	i_locator.access_in_cache(*imap)[ls] = tex;
+
+	return tex;
+}
+
 surface get_hexmask()
 {
 	static const image::locator terrain_mask(game_config::images::terrain_mask);
-	return get_image(terrain_mask, UNSCALED);
+	return get_surface(terrain_mask, UNSCALED);
+}
+
+point get_size(const locator& i_locator, bool skip_cache)
+{
+	const surface s(get_surface(i_locator, UNSCALED, skip_cache));
+	if (s != nullptr) {
+		return {s->w, s->h};
+	} else {
+		return {0, 0};
+	}
 }
 
 bool is_in_hex(const locator& i_locator)
@@ -951,7 +842,7 @@ bool is_in_hex(const locator& i_locator)
 		if(i_locator.in_cache(in_hex_info_)) {
 			result = i_locator.locate_in_cache(in_hex_info_);
 		} else {
-			const surface image(get_image(i_locator, UNSCALED));
+			const surface image(get_surface(i_locator, UNSCALED));
 
 			bool res = in_mask_surface(image, get_hexmask());
 
@@ -970,7 +861,7 @@ bool is_in_hex(const locator& i_locator)
 bool is_empty_hex(const locator& i_locator)
 {
 	if(!i_locator.in_cache(is_empty_hex_)) {
-		const surface surf = get_image(i_locator, HEXED);
+		const surface surf = get_surface(i_locator, HEXED);
 		// emptiness of terrain image is checked during hex cut
 		// so, maybe in cache now, let's recheck
 		if(!i_locator.in_cache(is_empty_hex_)) {
@@ -986,28 +877,6 @@ bool is_empty_hex(const locator& i_locator)
 	return i_locator.locate_in_cache(is_empty_hex_);
 }
 
-surface reverse_image(const surface& surf)
-{
-	if(surf == nullptr) {
-		return surface(nullptr);
-	}
-
-	const auto itor = reversed_images_.find(surf);
-	if(itor != reversed_images_.end()) {
-		// sdl_add_ref(itor->second);
-		return itor->second;
-	}
-
-	const surface rev(flip_surface(surf));
-	if(rev == nullptr) {
-		return surface(nullptr);
-	}
-
-	reversed_images_.emplace(surf, rev);
-	// sdl_add_ref(rev);
-	return rev;
-}
-
 bool exists(const image::locator& i_locator)
 {
 	typedef image::locator loc;
@@ -1080,7 +949,7 @@ bool precached_file_exists(const std::string& file)
 
 save_result save_image(const locator& i_locator, const std::string& filename)
 {
-	return save_image(get_image(i_locator), filename);
+	return save_image(get_surface(i_locator), filename);
 }
 
 save_result save_image(const surface& surf, const std::string& filename)
@@ -1090,26 +959,95 @@ save_result save_image(const surface& surf, const std::string& filename)
 	}
 
 	if(filesystem::ends_with(filename, ".jpeg") || filesystem::ends_with(filename, ".jpg") || filesystem::ends_with(filename, ".jpe")) {
-		LOG_DP << "Writing a JPG image to " << filename << std::endl;
+		LOG_IMG << "Writing a JPG image to " << filename;
 
 		const int err = IMG_SaveJPG_RW(surf, filesystem::make_write_RWops(filename).release(), true, 75); // SDL takes ownership of the RWops
 		return err == 0 ? save_result::success : save_result::save_failed;
 	}
 
 	if(filesystem::ends_with(filename, ".png")) {
-		LOG_DP << "Writing a PNG image to " << filename << std::endl;
+		LOG_IMG << "Writing a PNG image to " << filename;
 
 		const int err = IMG_SavePNG_RW(surf, filesystem::make_write_RWops(filename).release(), true); // SDL takes ownership of the RWops
 		return err == 0 ? save_result::success : save_result::save_failed;
 	}
 
-	if(filesystem::ends_with(filename, ".bmp")) {
-		LOG_DP << "Writing a BMP image to " << filename << std::endl;
-		const int err = SDL_SaveBMP(surf, filename.c_str()) == 0;
-		return err == 0 ? save_result::success : save_result::save_failed;
-	}
-
 	return save_result::unsupported_format;
 }
 
+/*
+ * TEXTURE INTERFACE ======================================================================
+ *
+ * The only important difference here is that textures must have their
+ * scale quality set before creation. All other handling is done by
+ * get_surface.
+ */
+
+texture get_texture(const image::locator& i_locator, TYPE type, bool skip_cache)
+{
+	return get_texture(i_locator, scale_quality::nearest, type, skip_cache);
+}
+
+/** Returns a texture for the corresponding image. */
+texture get_texture(const image::locator& i_locator, scale_quality quality, TYPE type, bool skip_cache)
+{
+	texture res;
+
+	if(i_locator.is_void()) {
+		return res;
+	}
+
+	type = simplify_type(i_locator, type);
+
+	//
+	// Select the appropriate cache. We don't need caches for every single image types,
+	// since some types can be handled by render-time operations.
+	//
+	texture_cache* cache = nullptr;
+
+	switch(type) {
+	case HEXED:
+		cache = &textures_hexed_[quality];
+		break;
+	case TOD_COLORED:
+		cache = &texture_tod_colored_[quality];
+		break;
+	default:
+		cache = &textures_[quality];
+	}
+
+	//
+	// Now attempt to find a cached texture. If found, return it.
+	//
+	if(auto cached_item = i_locator.copy_from_cache(*cache)) {
+		return *cached_item;
+	}
+
+	DBG_IMG << "texture cache [" << type << "] miss: " << i_locator;
+
+	//
+	// No texture was cached. In that case, create a new one. The explicit cases require special
+	// handling with surfaces in order to generate the desired effect. This shouldn't be the case
+	// once we get OGL and shader support.
+	//
+
+	// Get it from the surface cache, also setting the desired scale quality.
+	const bool linear_scaling = quality == scale_quality::linear ? true : false;
+	if(i_locator.get_modifications().empty()) {
+		// skip cache if we're loading plain files with no modifications
+		res = texture(get_surface(i_locator, type, true), linear_scaling);
+	} else {
+		res = texture(get_surface(i_locator, type, skip_cache), linear_scaling);
+	}
+
+	// Cache the texture.
+	if(skip_cache) {
+		DBG_IMG << "texture cache [" << type << "] skip: " << i_locator;
+	} else {
+		i_locator.add_to_cache(*cache, res);
+	}
+
+	return res;
+}
+
 } // end namespace image
