@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -18,9 +18,12 @@
 #include "map/location.hpp"
 #include "terrain/translation.hpp"
 
+#include <optional>
 #include <unordered_map>
 
 class surface;
+class texture;
+struct point;
 
 /**
  * Functions to load and save images from/to disk.
@@ -62,32 +65,36 @@ class locator
 public:
 	enum type { NONE, FILE, SUB_FILE };
 
-	locator();
-	locator(const locator& a, const std::string& mods = "");
-	locator(const char* filename);
-	locator(const std::string& filename);
-	locator(const std::string& filename, const std::string& modifications);
-	locator(const std::string& filename, const map_location& loc, int center_x, int center_y, const std::string& modifications = "");
+	locator() = default;
+	locator(locator&&) noexcept = default;
+	locator(const locator&) = default;
 
-	locator& operator=(const locator& a);
+	template<typename... Args>
+	locator(Args&&... args) : val_(std::forward<Args>(args)...)
+	{
+	}
 
-	bool operator==(const locator& a) const { return index_ == a.index_; }
-	bool operator!=(const locator& a) const { return index_ != a.index_; }
-	bool operator<(const locator& a) const { return index_ < a.index_; }
+	locator& operator=(const locator& a) = default;
+	locator& operator=(locator&&) = default;
 
-	const std::string& get_filename() const { return val_.filename_; }
-	bool is_data_uri() const { return val_.is_data_uri_; }
-	const map_location& get_loc() const { return val_.loc_ ; }
-	int get_center_x() const { return val_.center_x_; }
-	int get_center_y() const { return val_.center_y_; }
-	const std::string& get_modifications() const {return val_.modifications_;}
-	type get_type() const { return val_.type_; }
-	// const int get_index() const { return index_; };
+	/** Returns a copy of this locator with the given IPF */
+	locator clone(const std::string& mods) const;
+
+	bool operator==(const locator& a) const { return val_ == a.val_; }
+	bool operator!=(const locator& a) const { return !operator==(a); }
+
+	const std::string& get_filename() const { return val_.filename; }
+	bool is_data_uri() const { return val_.is_data_uri; }
+	const map_location& get_loc() const { return val_.loc ; }
+	int get_center_x() const { return val_.center_x; }
+	int get_center_y() const { return val_.center_y; }
+	const std::string& get_modifications() const { return val_.modifications; }
+	type get_type() const { return val_.type; }
 
 	/**
 	 * Returns @a true if the locator does not correspond to an actual image.
 	 */
-	bool is_void() const { return val_.type_ == NONE; }
+	bool is_void() const { return val_.type == NONE; }
 
 	/**
 	 * Tests whether the file the locator points at exists.
@@ -112,47 +119,48 @@ public:
 	const T& locate_in_cache(cache_type<T>& cache) const;
 
 	template<typename T>
-	void add_to_cache(cache_type<T>& cache, const T& data) const;
+	std::optional<T> copy_from_cache(cache_type<T>& cache) const;
+
+	template<typename T>
+	void add_to_cache(cache_type<T>& cache, T data) const;
 
 private:
-	// Called by each constructor after actual construction to
-	// initialize the index_ field
-	void init_index();
-	void parse_arguments();
-
 	struct value
 	{
-		value();
-		value(const char *filename);
+		value() = default;
+
 		value(const std::string& filename);
 		value(const std::string& filename, const std::string& modifications);
-		value(const std::string& filename, const map_location& loc, int center_x, int center_y, const std::string& modifications);
+		value(const std::string& filename, const map_location& loc, int center_x, int center_y, const std::string& modifications = "");
 
 		bool operator==(const value& a) const;
 		bool operator<(const value& a) const;
 
-		type type_;
-		bool is_data_uri_;
-		std::string filename_;
-		map_location loc_;
-		std::string modifications_;
-		int center_x_;
-		int center_y_;
+		locator::type type = NONE;
+		bool is_data_uri = false;
+		std::string filename{};
+		std::string modifications{};
+		map_location loc{};
+		int center_x = 0;
+		int center_y = 0;
 	};
 
-public:
-	typedef std::unordered_map<value, int> locator_finder_t;
+	value val_;
 
-private:
+public:
 	friend struct std::hash<value>;
 
-	int index_;
-	value val_;
+	template<typename T>
+	friend class cache_type;
+
+	std::size_t hash() const;
 };
 
-surface load_from_disk(const locator& loc);
+// write a readable representation of a locator, mostly for debugging
+std::ostream& operator<<(std::ostream&, const locator&);
 
-typedef cache_type<surface> image_cache;
+typedef cache_type<surface> surface_cache;
+typedef cache_type<texture> texture_cache;
 typedef cache_type<bool> bool_cache;
 
 typedef std::map<t_translation::terrain_code, surface> mini_terrain_cache_map;
@@ -176,10 +184,12 @@ extern mini_terrain_cache_map mini_highlighted_terrain_cache;
 typedef std::basic_string<signed char> light_string;
 
 /** Type used to pair light possibilities with the corresponding lit surface. */
-typedef std::map<light_string, surface> lit_variants;
+typedef std::map<light_string, surface> lit_surface_variants;
+typedef std::map<light_string, texture> lit_texture_variants;
 
 /** Lit variants for each locator. */
-typedef cache_type<lit_variants> lit_cache;
+typedef cache_type<lit_surface_variants> lit_surface_cache;
+typedef cache_type<lit_texture_variants> lit_texture_cache;
 
 /**
  * Returns the light_string for one light operation.
@@ -208,21 +218,12 @@ struct manager
 /**
  * Changes Time of Day color tint for all applicable image types.
  *
- * In particular this affects TOD_COLORED and BRIGHTENED images, as well as
+ * In particular this affects TOD_COLORED images, as well as
  * images with lightmaps applied. Changing the previous values automatically
- * invalidates all cached images of those types. It also invalidates the
- * internal cache used by reverse_image() (FIXME?).
+ * invalidates all cached images of those types.
  */
 void set_color_adjustment(int r, int g, int b);
 
-/**
- * Sets the scaling factor for images.
- *
- * Changing the previous value automatically invalidates all cached scaled
- * images.
- */
-void set_zoom(unsigned int zoom);
-
 /**
  * Used to specify the rendering format of images.
  */
@@ -230,40 +231,77 @@ enum TYPE
 {
 	/** Unmodified original-size image. */
 	UNSCALED,
-	/** Image rescaled according to the zoom settings. */
-	SCALED_TO_ZOOM,
 	/** Standard hexagonal tile mask applied, removing portions that don't fit. */
 	HEXED,
-	/** Image rescaled to fit into a hexagonal tile according to the zoom settings. */
-	SCALED_TO_HEX,
-	/** Same as SCALED_TO_HEX, but with Time of Day color tint applied. */
+	/** Same as HEXED, but with Time of Day color tint applied. */
 	TOD_COLORED,
-	/** Same as TOD_COLORED, but also brightened. */
-	BRIGHTENED
+	NUM_TYPES // Equal to the number of types specified above
 };
 
+enum class scale_quality { nearest, linear };
+
 /**
- * Caches and returns an image.
+ * Returns an image surface suitable for software manipulation.
+ *
+ * The equivalent get_texture() function should generally be preferred.
+ *
+ * Surfaces will be cached for repeat access, unless skip_cache is set.
  *
  * @param i_locator            Image path.
  * @param type                 Rendering format.
+ * @param skip_cache           Skip adding the result to the surface cache.
  */
-surface get_image(const locator& i_locator, TYPE type = UNSCALED);
+surface get_surface(const locator& i_locator, TYPE type = UNSCALED,
+	bool skip_cache = false);
+
+/**
+ * Returns an image texture suitable for hardware-accelerated rendering.
+ *
+ * Texture pointers are not unique, and will be cached and retained
+ * until no longer needed. Users of the returned texture do not have to
+ * worry about texture management.
+ *
+ * If caching is disabled via @a skip_cache, texture memory will be
+ * automatically freed once the returned object and all other linked
+ * textures (if any) are destroyed.
+ *
+ * @param i_locator            Image path.
+ * @param type                 Rendering format.
+ * @param skip_cache           Skip adding the result to the surface cache.
+ */
+texture get_texture(const locator& i_locator, TYPE type = UNSCALED,
+	bool skip_cache = false);
+
+texture get_texture(const image::locator& i_locator, scale_quality quality,
+	TYPE type = UNSCALED, bool skip_cache = false);
 
 /**
  * Caches and returns an image with a lightmap applied to it.
  *
+ * Images will always be HEXED type.
+ *
  * @param i_locator            Image path.
  * @param ls                   Light map to apply to the image.
- * @param type                 This should be either HEXED or SCALED_TO_HEX.
  */
-surface get_lighted_image(const image::locator& i_locator, const light_string& ls, TYPE type);
+surface get_lighted_image(const image::locator& i_locator, const light_string& ls);
+texture get_lighted_texture(const image::locator& i_locator, const light_string& ls);
 
 /**
  * Retrieves the standard hexagonal tile mask.
  */
 surface get_hexmask();
 
+/**
+ * Returns the width and height of an image.
+ *
+ * If the image is not yet in the surface cache, it will be loaded and cached
+ * unless skip_cache is explicitly set.
+ *
+ * @param i_locator            Image path.
+ * @param skip_cache           If true, do not cache the image if loaded.
+ */
+point get_size(const locator& i_locator, bool skip_cache = false);
+
 /**
  * Checks if an image fits into a single hex.
  */
@@ -277,14 +315,6 @@ bool is_in_hex(const locator& i_locator);
  */
 bool is_empty_hex(const locator& i_locator);
 
-/**
- * Horizontally flips an image.
- *
- * The input MUST have originally been returned from an image namespace function.
- * Returned images have the same semantics as those obtained from get_image().
- */
-surface reverse_image(const surface& surf);
-
 /**
  * Returns @a true if the given image actually exists, without loading it.
  */
