@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -36,6 +36,7 @@
 #include "units/unit.hpp"
 #include "units/helper.hpp"
 #include "units/types.hpp"
+#include "units/unit_alignments.hpp"
 #include "whiteboard/manager.hpp"
 
 #include <cassert>
@@ -82,7 +83,7 @@ static config image_report(const std::string &image,
 using font::span_color;
 
 static void add_status(config &r,
-	char const *path, char const *desc1, char const *desc2)
+	const std::string& path, char const *desc1, char const *desc2)
 {
 	std::ostringstream s;
 	s << translation::gettext(desc1) << translation::gettext(desc2);
@@ -96,7 +97,7 @@ static std::string flush(std::ostringstream &s)
 	return r;
 }
 
-static const time_of_day get_visible_time_of_day_at(reports::context & rc, const map_location & hex)
+static const time_of_day get_visible_time_of_day_at(const reports::context& rc, const map_location & hex)
 {
 	const team &viewing_team = rc.teams()[rc.screen().viewing_team()];
 	if (viewing_team.shrouded(hex)) {
@@ -122,34 +123,34 @@ struct report_generator_helper
 };
 
 #define REPORT_GENERATOR(n, cn) \
-	static config report_##n(reports::context & cn); \
+	static config report_##n(const reports::context& cn); \
 	static report_generator_helper reg_gen_##n(#n, &report_##n); \
-	static config report_##n(reports::context & cn)
+	static config report_##n(const reports::context& cn)
 
 static char const *naps = "</span>";
 
-static const unit *get_visible_unit(reports::context & rc)
+static const unit *get_visible_unit(const reports::context& rc)
 {
 	return rc.dc().get_visible_unit(rc.screen().displayed_unit_hex(),
 		rc.teams()[rc.screen().viewing_team()],
 		rc.screen().show_everything());
 }
 
-static const unit *get_selected_unit(reports::context & rc)
+static const unit *get_selected_unit(const reports::context& rc)
 {
 	return rc.dc().get_visible_unit(rc.screen().selected_hex(),
 		rc.teams()[rc.screen().viewing_team()],
 		rc.screen().show_everything());
 }
 
-static unit_const_ptr get_selected_unit_ptr(reports::context & rc)
+static unit_const_ptr get_selected_unit_ptr(const reports::context& rc)
 {
 	return rc.dc().get_visible_unit_shared_ptr(rc.screen().selected_hex(),
 		rc.teams()[rc.screen().viewing_team()],
 		rc.screen().show_everything());
 }
 
-static config gray_inactive(reports::context & rc, const std::string &str, const std::string& tooltip = "")
+static config gray_inactive(const reports::context& rc, const std::string &str, const std::string& tooltip = "")
 {
 	if ( rc.screen().viewing_side() == rc.screen().playing_side() )
 			return text_report(str, tooltip);
@@ -241,7 +242,7 @@ static std::string side_tooltip(const team& team)
 			 {"color_name", team::get_side_color_name_for_UI(team.side()) }});
 }
 
-static config unit_side(reports::context & rc, const unit* u)
+static config unit_side(const reports::context& rc, const unit* u)
 {
 	if (!u) return config();
 
@@ -307,7 +308,7 @@ static config unit_traits(const unit* u)
 	config res;
 	const std::vector<t_string> &traits = u->trait_names();
 	const std::vector<t_string> &descriptions = u->trait_descriptions();
-	const std::vector<std::string> &trait_ids = u->get_traits_list();
+	const std::vector<std::string> &trait_ids = u->trait_nonhidden_ids();
 	unsigned nb = traits.size();
 	for (unsigned i = 0; i < nb; ++i)
 	{
@@ -331,7 +332,7 @@ REPORT_GENERATOR(selected_unit_traits, rc)
 	return unit_traits(u);
 }
 
-static config unit_status(reports::context & rc, const unit* u)
+static config unit_status(const reports::context& rc, const unit* u)
 {
 	if (!u) return config();
 	config res;
@@ -352,6 +353,14 @@ static config unit_status(reports::context & rc, const unit* u)
 		add_status(res, "misc/petrified.png", N_("petrified: "),
 			N_("This unit has been petrified. It may not move or attack."));
 	}
+	if (u->get_state(unit::STATE_UNHEALABLE)) {
+		add_status(res, "misc/unhealable.png", N_("unhealable: "),
+			N_("This unit is unhealable. It cannot be healed by healers or villages and doesn’t benefit from resting."));
+	}
+	if (u->get_state(unit::STATE_INVULNERABLE)) {
+		add_status(res, "misc/invulnerable.png", N_("invulnerable: "),
+			N_("This unit is invulnerable. It cannot be harmed by any attack."));
+	}
 	return res;
 }
 REPORT_GENERATOR(unit_status,rc)
@@ -365,12 +374,12 @@ REPORT_GENERATOR(selected_unit_status, rc)
 	return unit_status(rc, u);
 }
 
-static config unit_alignment(reports::context & rc, const unit* u, const map_location& hex)
+static config unit_alignment(const reports::context& rc, const unit* u, const map_location& hex)
 {
 	if (!u) return config();
 	std::ostringstream str, tooltip;
 	const std::string align = unit_type::alignment_description(u->alignment(), u->gender());
-	const std::string align_id = u->alignment().to_string();
+	const std::string align_id = unit_alignments::get_string(u->alignment());
 	const time_of_day effective_tod = get_visible_time_of_day_at(rc, hex);
 	int cm = combat_modifier(effective_tod, u->alignment(), u->is_fearless());
 
@@ -443,9 +452,12 @@ static config unit_abilities(const unit* u, const map_location& loc)
 REPORT_GENERATOR(unit_abilities, rc)
 {
 	const unit *u = get_visible_unit(rc);
+	const team &viewing_team = rc.teams()[rc.screen().viewing_team()];
 	const map_location& mouseover_hex = rc.screen().mouseover_hex();
+	const map_location& displayed_unit_hex = rc.screen().displayed_unit_hex();
+	const map_location& hex = (mouseover_hex.valid() && !viewing_team.shrouded(mouseover_hex)) ? mouseover_hex : displayed_unit_hex;
 
-	return unit_abilities(u, mouseover_hex);
+	return unit_abilities(u, hex);
 }
 REPORT_GENERATOR(selected_unit_abilities, rc)
 {
@@ -453,25 +465,27 @@ REPORT_GENERATOR(selected_unit_abilities, rc)
 
 	const map_location& mouseover_hex = rc.screen().mouseover_hex();
 	const unit *visible_unit = get_visible_unit(rc);
-	if(visible_unit && u && visible_unit->id() != u->id() && mouseover_hex.valid())
+	const team &viewing_team = rc.teams()[rc.screen().viewing_team()];
+
+	if (visible_unit && u && visible_unit->id() != u->id() && mouseover_hex.valid() && !viewing_team.shrouded(mouseover_hex))
 		return unit_abilities(u, mouseover_hex);
 	else
 		return unit_abilities(u, u->get_location());
 }
 
 
-static config unit_hp(reports::context& rc, const unit* u)
+static config unit_hp(const reports::context& rc, const unit* u)
 {
 	if (!u) return config();
 	std::ostringstream str, tooltip;
 	str << span_color(u->hp_color()) << u->hitpoints()
 		<< '/' << u->max_hitpoints() << naps;
 
-	std::set<std::string> resistances_table;
+	std::vector<std::string> resistances_table;
 
 	bool att_def_diff = false;
 	map_location displayed_unit_hex = rc.screen().displayed_unit_hex();
-	for (const utils::string_map::value_type &resist : u->get_base_resistances())
+	for (const utils::string_map_res::value_type &resist : u->get_base_resistances())
 	{
 		std::ostringstream line;
 		line << translation::gettext(resist.first.c_str()) << ": ";
@@ -490,7 +504,7 @@ static config unit_hp(reports::context& rc, const unit* u)
 			<< naps << '\n';
 			att_def_diff = true;
 		}
-		resistances_table.insert(line.str());
+		resistances_table.push_back(line.str());
 	}
 
 	tooltip << _("Resistances: ");
@@ -560,7 +574,7 @@ REPORT_GENERATOR(selected_unit_advancement_options, rc)
 	return unit_advancement_options(u);
 }
 
-static config unit_defense(reports::context & rc, const unit* u, const map_location& displayed_unit_hex)
+static config unit_defense(const reports::context& rc, const unit* u, const map_location& displayed_unit_hex)
 {
 	if(!u) {
 		return config();
@@ -654,7 +668,7 @@ REPORT_GENERATOR(selected_unit_vision, rc)
 	return unit_vision(u);
 }
 
-static config unit_moves(reports::context & rc, const unit* u, bool is_visible_unit)
+static config unit_moves(const reports::context& rc, const unit* u, bool is_visible_unit)
 {
 	if (!u) return config();
 	std::ostringstream str, tooltip;
@@ -683,23 +697,30 @@ static config unit_moves(reports::context & rc, const unit* u, bool is_visible_u
 	for (const terrain_movement& tm : terrain_moves) {
 		tooltip << tm.name << ": ";
 
-		std::string color;
 		//movement  -  range: 1 .. 5, movetype::UNREACHABLE=impassable
-		const bool cannot_move = tm.moves > u->total_movement();
-		if (cannot_move)		// cannot move in this terrain
-			color = "red";
-		else if (tm.moves > 1)
-			color = "yellow";
-		else
-			color = "white";
+		const bool cannot_move = tm.moves > u->total_movement();		// cannot move in this terrain
+		double movement_red_to_green = 100.0 - 25.0 * tm.moves;
+
+		// passing true to select the less saturated red-to-green scale
+		std::string color = game_config::red_to_green(movement_red_to_green, true).to_hex_string();
 		tooltip << "<span foreground=\"" << color << "\">";
 		// A 5 MP margin; if the movement costs go above
 		// the unit's max moves + 5, we replace it with dashes.
 		if (cannot_move && (tm.moves > u->total_movement() + 5)) {
 			tooltip << font::unicode_figure_dash;
+		} else if (cannot_move) {
+			tooltip << "(" << tm.moves << ")";
 		} else {
 			tooltip << tm.moves;
 		}
+		if(tm.moves != 0) {
+			const int movement_hexes_per_turn = u->total_movement() / tm.moves;
+			tooltip << " ";
+			for(int i = 0; i < movement_hexes_per_turn; ++i) {
+				// Unicode horizontal black hexagon and Unicode zero width space (to allow a line break)
+				tooltip << "\u2b23\u200b";
+			}
+		}
 		tooltip << naps << '\n';
 	}
 
@@ -740,15 +761,20 @@ REPORT_GENERATOR(selected_unit_moves, rc)
 	return unit_moves(rc, u, false);
 }
 
+/**
+ * Maps resistance <= -60 (resistance value <= -60%) to intense red.
+ * Maps resistance >= 60 (resistance value >= 60%) to intense green.
+ * Intermediate values are affinely mapped to the red-to-green scale,
+ * with 0 (0%) being mapped to yellow.
+ * Compare unit_helper::resistance_color().
+ */
 static inline const color_t attack_info_percent_color(int resistance)
 {
-	// Compare unit_helper::resistance_color()
-	if (resistance < 0) return font::BAD_COLOR;
-	if (resistance > 0) return font::GOOD_COLOR;
-	return font::YELLOW_COLOR;
+	// Passing false to select the more saturated red-to-green scale.
+	return game_config::red_to_green(50.0 + resistance * 5.0 / 6.0, false);
 }
 
-static int attack_info(reports::context & rc, const attack_type &at, config &res, const unit &u, const map_location &hex, const unit* sec_u = nullptr, const_attack_ptr sec_u_weapon = nullptr)
+static int attack_info(const reports::context& rc, const attack_type &at, config &res, const unit &u, const map_location &hex, const unit* sec_u = nullptr, const_attack_ptr sec_u_weapon = nullptr)
 {
 	std::ostringstream str, tooltip;
 	int damage = 0;
@@ -761,7 +787,7 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 	{
 		auto ctx = at.specials_context(u.shared_from_this(), hex, u.side() == rc.screen().playing_side());
 		int base_damage = at.damage();
-		int specials_damage = at.modified_damage(false);
+		int specials_damage = at.modified_damage();
 		int damage_multiplier = 100;
 		const_attack_ptr weapon  = at.shared_from_this();
 		int tod_bonus = combat_modifier(get_visible_time_of_day_at(rc, hex), u.alignment(), u.is_fearless());
@@ -781,7 +807,7 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 
 		unsigned base_attacks = at.num_attacks();
 		unsigned min_attacks, max_attacks;
-		at.modified_attacks(false, min_attacks, max_attacks);
+		at.modified_attacks(min_attacks, max_attacks);
 		unsigned num_attacks = swarm_blows(min_attacks, max_attacks, cur_hp, max_hp);
 
 		color_t dmg_color = font::weapon_color;
@@ -850,15 +876,22 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 		const string_with_tooltip damage_and_num_attacks {flush(str), flush(tooltip)};
 
 		std::string range = string_table["range_" + at.range()];
-		std::string lang_type = string_table["type_" + at.type()];
+		std::pair<std::string, std::string> types = at.damage_type();
+		std::string secondary_lang_type = types.second;
+		if (!secondary_lang_type.empty()) {
+			secondary_lang_type = ", " + string_table["type_" + secondary_lang_type];
+		}
+		std::string lang_type = string_table["type_" + types.first] + secondary_lang_type;
 
-		// SCALE_INTO_SHARP() is needed in case the 72x72 images/misc/missing-image.png is substituted.
-		const std::string range_png = std::string("icons/profiles/") + at.range() + "_attack.png~SCALE_INTO_SHARP(16,16)";
-		const std::string type_png = std::string("icons/profiles/") + at.type() + ".png~SCALE_INTO_SHARP(16,16)";
+		// SCALE_INTO() is needed in case the 72x72 images/misc/missing-image.png is substituted.
+		const std::string range_png = std::string("icons/profiles/") + at.range() + "_attack.png~SCALE_INTO(16,16)";
+		const std::string type_png = std::string("icons/profiles/") + types.first + ".png~SCALE_INTO(16,16)";
+		const std::string secondary_type_png = !(types.second).empty() ? std::string("icons/profiles/") + types.second + ".png~SCALE_INTO(16,16)" : "";
 		const bool range_png_exists = image::locator(range_png).file_exists();
 		const bool type_png_exists = image::locator(type_png).file_exists();
+		const bool secondary_type_png_exists = image::locator(secondary_type_png).file_exists();
 
-		if(!range_png_exists || !type_png_exists) {
+		if(!range_png_exists || !type_png_exists || (!secondary_type_png_exists && !secondary_lang_type.empty())) {
 			str << span_color(font::weapon_details_color) << "  " << "  "
 				<< range << font::weapon_details_sep
 				<< lang_type << "</span>\n";
@@ -913,8 +946,11 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 
 		// The icons are 16x16. We add 5px padding for alignment reasons (placement of the icon in relation to ascender and descender letters).
 		const std::string spacer = "misc/blank.png~CROP(0, 0, 16, 21)"; // 21 == 16+5
-		if (range_png_exists) add_image(res, spacer + "~BLIT(" + range_png + ",0,5)", damage_versus.tooltip);
-		if (type_png_exists) add_image(res, spacer + "~BLIT(" + type_png + ",0,5)", damage_versus.tooltip);
+		add_image(res, spacer + "~BLIT(" + range_png + ",0,5)", damage_versus.tooltip);
+		add_image(res, spacer + "~BLIT(" + type_png + ",0,5)", damage_versus.tooltip);
+		if(secondary_type_png_exists){
+			add_image(res, spacer + "~BLIT(" + secondary_type_png + ",0,5)", damage_versus.tooltip);
+		}
 		add_text(res, damage_and_num_attacks.str, damage_and_num_attacks.tooltip);
 		add_text(res, damage_versus.str, damage_versus.tooltip); // This string is usually empty
 
@@ -996,7 +1032,7 @@ static std::string format_hp(unsigned hp)
 	return res.str();
 }
 
-static config unit_weapons(reports::context & rc, unit_const_ptr attacker, const map_location &attacker_pos, const unit *defender, bool show_attacker)
+static config unit_weapons(const reports::context& rc, unit_const_ptr attacker, const map_location &attacker_pos, const unit *defender, bool show_attacker)
 {
 	if (!attacker || !defender) return config();
 
@@ -1120,7 +1156,7 @@ static config unit_weapons(reports::context & rc, unit_const_ptr attacker, const
  * Display the attacks of the displayed unit against the unit passed as argument.
  * 'hex' is the location the attacker will be at during combat.
  */
-static config unit_weapons(reports::context & rc, const unit *u, const map_location &hex)
+static config unit_weapons(const reports::context& rc, const unit *u, const map_location &hex)
 {
 	config res = config();
 	if ((u != nullptr) && (!u->attacks().empty())) {
@@ -1220,7 +1256,7 @@ REPORT_GENERATOR(unit_profile, rc)
 	return image_report(u->small_profile());
 }
 
-static config tod_stats_at(reports::context & rc, const map_location& hex)
+static config tod_stats_at(const reports::context& rc, const map_location& hex)
 {
 	std::ostringstream tooltip;
 	std::ostringstream text;
@@ -1259,13 +1295,13 @@ REPORT_GENERATOR(selected_tod_stats, rc)
 	return tod_stats_at(rc, hex);
 }
 
-static config time_of_day_at(reports::context & rc, const map_location& mouseover_hex)
+static config time_of_day_at(const reports::context& rc, const map_location& mouseover_hex)
 {
 	std::ostringstream tooltip;
 	time_of_day tod = get_visible_time_of_day_at(rc, mouseover_hex);
 
 	int b = tod.lawful_bonus;
-	int l = generic_combat_modifier(b, unit_type::ALIGNMENT::LIMINAL, false, rc.tod().get_max_liminal_bonus());
+	int l = generic_combat_modifier(b, unit_alignments::type::liminal, false, rc.tod().get_max_liminal_bonus());
 	std::string  lawful_color("white");
 	std::string chaotic_color("white");
 	std::string liminal_color("white");
@@ -1311,14 +1347,14 @@ REPORT_GENERATOR(selected_time_of_day, rc)
 	return time_of_day_at(rc, hex);
 }
 
-static config unit_box_at(reports::context & rc, const map_location& mouseover_hex)
+static config unit_box_at(const reports::context& rc, const map_location& mouseover_hex)
 {
 	std::ostringstream tooltip;
 	time_of_day global_tod = rc.tod().get_time_of_day();
 	time_of_day local_tod = get_visible_time_of_day_at(rc, mouseover_hex);
 
 	int bonus = local_tod.lawful_bonus;
-	int l = generic_combat_modifier(bonus, unit_type::ALIGNMENT::LIMINAL, false, rc.tod().get_max_liminal_bonus());
+	int l = generic_combat_modifier(bonus, unit_alignments::type::liminal, false, rc.tod().get_max_liminal_bonus());
 
 	std::string  lawful_color("white");
 	std::string chaotic_color("white");
@@ -1508,24 +1544,27 @@ void blit_tced_icon(config &cfg, const std::string &terrain_id, const std::strin
 
 REPORT_GENERATOR(terrain_info, rc)
 {
-	const gamemap &map = rc.map();
+	const gamemap& map = rc.map();
 	map_location mouseover_hex = rc.screen().mouseover_hex();
 
-	if (!map.on_board(mouseover_hex))
+	if(!map.on_board(mouseover_hex)) {
 		mouseover_hex = rc.screen().selected_hex();
+	}
 
-	if (!map.on_board(mouseover_hex))
+	if(!map.on_board(mouseover_hex)) {
 		return config();
+	}
 
 	t_translation::terrain_code terrain = map.get_terrain(mouseover_hex);
-	if (t_translation::terrain_matches(terrain, t_translation::ALL_OFF_MAP))
+	if(t_translation::terrain_matches(terrain, t_translation::ALL_OFF_MAP)) {
 		return config();
+	}
 
 	config cfg;
 
 	bool high_res = false;
 
-	if (display::get_singleton()->shrouded(mouseover_hex)) {
+	if(display::get_singleton()->shrouded(mouseover_hex)) {
 		return cfg;
 	}
 	//TODO
@@ -1538,17 +1577,46 @@ REPORT_GENERATOR(terrain_info, rc)
 //	}
 
 	const t_translation::ter_list& underlying_terrains = map.underlying_union_terrain(terrain);
-	for (const t_translation::terrain_code& underlying_terrain : underlying_terrains) {
-
-		if (t_translation::terrain_matches(underlying_terrain, t_translation::ALL_OFF_MAP))
+	for(const t_translation::terrain_code& underlying_terrain : underlying_terrains) {
+		if(t_translation::terrain_matches(underlying_terrain, t_translation::ALL_OFF_MAP)) {
 			continue;
+		}
 		const std::string& terrain_id = map.get_terrain_info(underlying_terrain).id();
 		const std::string& terrain_name = map.get_terrain_string(underlying_terrain);
 		const std::string& terrain_icon = map.get_terrain_info(underlying_terrain).icon_image();
-		if (terrain_icon.empty())
+		if(terrain_icon.empty()) {
 			continue;
+		}
 		blit_tced_icon(cfg, terrain_id, terrain_icon, high_res, terrain_name);
 	}
+
+	if(map.is_village(mouseover_hex)) {
+		int owner = rc.dc().village_owner(mouseover_hex);
+		// This report is used in both game and editor. get_team(viewing_side) would throw in the editor's
+		// terrain-only mode, but if the village already has an owner then we're not in that mode.
+		if(owner != 0) {
+			int viewing_side = rc.screen().viewing_side();
+			const team& viewing_team = rc.dc().get_team(viewing_side);
+
+			if(!viewing_team.fogged(mouseover_hex)) {
+				const team& owner_team = rc.dc().get_team(owner);
+
+				std::string flag_icon = owner_team.flag_icon();
+				std::string old_rgb = game_config::flag_rgb;
+				std::string new_rgb = team::get_side_color_id(owner_team.side());
+				std::string mods = "~RC(" + old_rgb + ">" + new_rgb + ")";
+				if(flag_icon.empty()) {
+					flag_icon = game_config::images::flag_icon;
+				}
+				std::string tooltip = side_tooltip(owner_team);
+				std::string side = std::to_string(owner_team.side());
+
+				add_image(cfg, flag_icon + mods, tooltip);
+				add_text(cfg, side, tooltip);
+			}
+		}
+	}
+
 	return cfg;
 }
 
@@ -1669,28 +1737,6 @@ REPORT_GENERATOR(observers, rc)
 	return image_report(game_config::images::observer, str.str());
 }
 
-/* TODO unused
-REPORT_GENERATOR(selected_terrain)
-{
-	const std::string selected_terrain = editor::get_selected_terrain();
-	if (selected_terrain.empty())
-		return config();
-	else
-		return text_report(selected_terrain);
-}
-*/
-
-/* TODO this is unused
-REPORT_GENERATOR(edit_left_button_function)
-{
-	const std::string left_button_function = editor::get_left_button_function();
-	if (left_button_function.empty())
-		return config();
-	else
-		return text_report(left_button_function);
-}
-*/
-
 REPORT_GENERATOR(report_clock, /*rc*/)
 {
 	config report;
@@ -1756,7 +1802,7 @@ void reports::register_generator(const std::string &name, reports::generator *g)
 	dynamic_generators_[name].reset(g);
 }
 
-config reports::generate_report(const std::string &name, reports::context & rc, bool only_static)
+config reports::generate_report(const std::string &name, const reports::context& rc, bool only_static)
 {
 	if (!only_static) {
 		dynamic_report_generators::const_iterator i = dynamic_generators_.find(name);
