@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2016 - 2022
+	Copyright (C) 2016 - 2024
 	by Iris Morelle <shadowm2006@gmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -74,7 +74,7 @@ const platform_file_handle_type INVALID_FILE_HANDLE =
 inline void atomic_fail(const std::string& step_description)
 {
 	const std::string errno_desc = std::strerror(errno);
-	ERR_FS << "Atomic commit failed (" << step_description << "): " << errno_desc << '\n';
+	ERR_FS << "Atomic commit failed (" << step_description << "): " << errno_desc;
 	throw filesystem::io_exception(std::string("Atomic commit failed (") + step_description + ")");
 }
 
@@ -102,7 +102,7 @@ platform_file_handle_type get_stream_file_descriptor(std::ostream& os)
  */
 filesystem::scoped_ostream ostream_file_with_delete(const std::string& fname)
 {
-	LOG_FS << "streaming " << fname << " for writing with delete access.\n";
+	LOG_FS << "streaming " << fname << " for writing with delete access.";
 
 	namespace bfs = boost::filesystem;
 	const auto& w_name = unicode_cast<std::wstring>(fname);
@@ -146,7 +146,7 @@ filesystem::scoped_ostream ostream_file_with_delete(const std::string& fname)
 bool rename_open_file(const std::string& new_name, HANDLE open_handle)
 {
 	if(open_handle == INVALID_HANDLE_VALUE) {
-		ERR_FS << "replace_open_file(): Bad handle\n";
+		ERR_FS << "replace_open_file(): Bad handle";
 		return false;
 	}
 
@@ -171,7 +171,7 @@ bool rename_open_file(const std::string& new_name, HANDLE open_handle)
 								   fileinfo_buf.get(),
 								   static_cast<DWORD>(buf_size)))
 	{
-		ERR_FS << "replace_open_file(): SetFileInformationByHandle() " << GetLastError() << '\n';
+		ERR_FS << "replace_open_file(): SetFileInformationByHandle() " << GetLastError();
 		return false;
 	}
 
@@ -193,20 +193,20 @@ atomic_commit::atomic_commit(const std::string& filename)
 	, handle_(filesystem::get_stream_file_descriptor(*out_))
 #endif
 {
-	LOG_FS << "Atomic write guard created for " << dest_name_ << " using " << temp_name_ << '\n';
+	LOG_FS << "Atomic write guard created for " << dest_name_ << " using " << temp_name_;
 }
 
 atomic_commit::~atomic_commit()
 {
 	if(!temp_name_.empty()) {
-		ERR_FS << "Temporary file for atomic write leaked: " << temp_name_ << '\n';
+		ERR_FS << "Temporary file for atomic write leaked: " << temp_name_;
 	}
 }
 
 void atomic_commit::commit()
 {
 	if(temp_name_.empty()) {
-		ERR_FS << "Attempted to commit " << dest_name_ << " more than once!\n";
+		ERR_FS << "Attempted to commit " << dest_name_ << " more than once!";
 		return;
 	}
 
@@ -224,7 +224,7 @@ void atomic_commit::commit()
 	}
 #endif
 
-	LOG_FS << "Atomic commit succeeded: " << temp_name_ << " -> " << dest_name_ << '\n';
+	LOG_FS << "Atomic commit succeeded: " << temp_name_ << " -> " << dest_name_;
 
 	temp_name_.clear();
 }
