@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -17,19 +17,26 @@
 
 #include "show_dialog.hpp"
 
+#include "draw.hpp"
+#include "draw_manager.hpp"
 #include "floating_label.hpp"
 #include "picture.hpp"
 #include "gettext.hpp"
-#include "gui/core/event/handler.hpp"
+#include "gui/core/event/handler.hpp" // is_in_dialog
 #include "help/help.hpp"
 #include "hotkey/command_executor.hpp"
 #include "log.hpp"
 #include "font/sdl_ttf_compat.hpp"
 #include "font/standard_colors.hpp"
 #include "sdl/rect.hpp"
+#include "sdl/input.hpp" // get_mouse_state
+#include "sdl/utils.hpp" // blur_surface
+#include "video.hpp"
 
 static lg::log_domain log_display("display");
 #define ERR_DP LOG_STREAM(err, log_display)
+#define WRN_DP LOG_STREAM(warn, log_display)
+#define DBG_DP LOG_STREAM(debug, log_display)
 #define ERR_G  LOG_STREAM(err, lg::general)
 
 namespace {
@@ -64,7 +71,7 @@ dialog_manager::~dialog_manager()
 {
 	is_in_dialog = reset_to;
 	int mousex, mousey;
-	SDL_GetMouseState(&mousex, &mousey);
+	sdl::get_mouse_state(&mousex, &mousey);
 	SDL_Event pb_event;
 	pb_event.type = SDL_MOUSEMOTION;
 	pb_event.motion.state = 0;
@@ -75,48 +82,55 @@ dialog_manager::~dialog_manager()
 	SDL_PushEvent(&pb_event);
 }
 
-dialog_frame::dialog_frame(CVideo& video, const std::string& title,
-		const style& style, bool auto_restore,
+dialog_frame::dialog_frame(const std::string& title,
+		const style& style,
 		std::vector<button*>* buttons, button* help_button) :
 	title_(title),
-	video_(video),
 	dialog_style_(style),
 	buttons_(buttons),
 	help_button_(help_button),
-	restorer_(nullptr),
-	auto_restore_(auto_restore),
 	dim_(),
-	top_(image::get_image("dialogs/" + dialog_style_.panel + "-border-top.png")),
-	bot_(image::get_image("dialogs/" + dialog_style_.panel + "-border-bottom.png")),
-	left_(image::get_image("dialogs/" + dialog_style_.panel + "-border-left.png")),
-	right_(image::get_image("dialogs/" + dialog_style_.panel + "-border-right.png")),
-	top_left_(image::get_image("dialogs/" + dialog_style_.panel + "-border-topleft.png")),
-	bot_left_(image::get_image("dialogs/" + dialog_style_.panel + "-border-botleft.png")),
-	top_right_(image::get_image("dialogs/" + dialog_style_.panel + "-border-topright.png")),
-	bot_right_(image::get_image("dialogs/" + dialog_style_.panel + "-border-botright.png")),
-	bg_(image::get_image("dialogs/" + dialog_style_.panel + "-background.png")),
-	have_border_(top_ != nullptr && bot_ != nullptr && left_ != nullptr && right_ != nullptr),
+	top_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-top.png")),
+	bot_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-bottom.png")),
+	left_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-left.png")),
+	right_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-right.png")),
+	top_left_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-topleft.png")),
+	bot_left_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-botleft.png")),
+	top_right_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-topright.png")),
+	bot_right_(image::get_texture("dialogs/" + dialog_style_.panel + "-border-botright.png")),
+	bg_(image::get_texture("dialogs/" + dialog_style_.panel + "-background.png")),
+	have_border_(top_ && bot_ && left_ && right_),
 	dirty_(true)
 {
+	// Raise buttons so they are drawn on top.
+	// and BTW buttons_ being a pointer to a vector is fucking insane
+	for (button* b : *buttons_) {
+		draw_manager::raise_drawable(b);
+	}
+	if (help_button) {
+		draw_manager::raise_drawable(help_button_);
+	}
 }
 
 dialog_frame::~dialog_frame()
 {
-	delete restorer_;
+	draw_manager::invalidate_region(screen_location());
 }
 
 dialog_frame::dimension_measurements::dimension_measurements() :
 	interior(sdl::empty_rect), exterior(sdl::empty_rect), title(sdl::empty_rect), button_row(sdl::empty_rect)
 {}
 
-dialog_frame::dimension_measurements dialog_frame::layout(const SDL_Rect& rect) {
+dialog_frame::dimension_measurements dialog_frame::layout(const SDL_Rect& rect)
+{
 	return layout(rect.x, rect.y, rect.w, rect.h);
 }
 
-int dialog_frame::top_padding() const {
+int dialog_frame::top_padding() const
+{
 	int padding = 0;
 	if(have_border_) {
-		padding += top_->h;
+		padding += top_.h();
 	}
 	if(!title_.empty()) {
 		padding += font::get_max_height(font::SIZE_TITLE) + 2*dialog_frame::title_border_h;
@@ -124,40 +138,11 @@ int dialog_frame::top_padding() const {
 	return padding;
 }
 
-void dialog_frame::set_dirty(bool dirty) {
+void dialog_frame::set_dirty(bool dirty)
+{
 	dirty_ = dirty;
 }
 
-void dialog_frame::handle_window_event(const SDL_Event& event) {
-
-	if (event.type == SDL_WINDOWEVENT) {
-		switch (event.window.event) {
-		case SDL_WINDOWEVENT_RESIZED:
-		case SDL_WINDOWEVENT_RESTORED:
-		case SDL_WINDOWEVENT_SHOWN:
-		case SDL_WINDOWEVENT_EXPOSED:
-			set_dirty();
-		}
-	}
-}
-
-void dialog_frame::handle_event(const SDL_Event& event) {
-
-	if (event.type == DRAW_ALL_EVENT) {
-		set_dirty();
-
-		if (buttons_) {
-			for(std::vector<button *>::iterator it = buttons_->begin(); it != buttons_->end(); ++it) {
-				(*it)->set_dirty(true);
-			}
-		}
-	}
-
-	if (event.type == DRAW_EVENT || event.type == DRAW_ALL_EVENT) {
-		draw();
-	}
-}
-
 int dialog_frame::bottom_padding() const {
 	int padding = 0;
 	if(buttons_ != nullptr) {
@@ -166,7 +151,7 @@ int dialog_frame::bottom_padding() const {
 		}
 	}
 	if(have_border_) {
-		padding += bot_->h;
+		padding += bot_.h();
 	}
 	return padding;
 }
@@ -174,7 +159,7 @@ int dialog_frame::bottom_padding() const {
 dialog_frame::dimension_measurements dialog_frame::layout(int x, int y, int w, int h) {
 	dim_ = dimension_measurements();
 	if(!title_.empty()) {
-		dim_.title = draw_title(nullptr);
+		dim_.title = draw_title(false);
 		dim_.title.w += title_border_w;
 	}
 	if(buttons_ != nullptr) {
@@ -201,12 +186,12 @@ dialog_frame::dimension_measurements dialog_frame::layout(int x, int y, int w, i
 	h += dim_.title.h + dim_.button_row.h;
 	dim_.button_row.x += x + w;
 
-	SDL_Rect bounds = video_.screen_area();
+	rect bounds = video::game_canvas();
 	if(have_border_) {
-		bounds.x += left_->w;
-		bounds.y += top_->h;
-		bounds.w -= left_->w;
-		bounds.h -= top_->h;
+		bounds.x += left_.w();
+		bounds.y += top_.h();
+		bounds.w -= left_.w();
+		bounds.h -= top_.h();
 	}
 	if(x < bounds.x) {
 		w += x;
@@ -231,15 +216,18 @@ dialog_frame::dimension_measurements dialog_frame::layout(int x, int y, int w, i
 	dim_.interior.w = w;
 	dim_.interior.h = h;
 	if(have_border_) {
-		dim_.exterior.x = dim_.interior.x - left_->w;
-		dim_.exterior.y = dim_.interior.y - top_->h;
-		dim_.exterior.w = dim_.interior.w + left_->w + right_->w;
-		dim_.exterior.h = dim_.interior.h + top_->h + bot_->h;
+		dim_.exterior.x = dim_.interior.x - left_.w();
+		dim_.exterior.y = dim_.interior.y - top_.h();
+		dim_.exterior.w = dim_.interior.w + left_.w() + right_.w();
+		dim_.exterior.h = dim_.interior.h + top_.h() + bot_.h();
 	} else {
 		dim_.exterior = dim_.interior;
 	}
 	dim_.title.x = dim_.interior.x + title_border_w;
 	dim_.title.y = dim_.interior.y + title_border_h;
+
+	draw_manager::invalidate_region(dim_.exterior);
+
 	return dim_;
 }
 
@@ -249,88 +237,94 @@ void dialog_frame::draw_border()
 		return;
 	}
 
-	surface top_image(scale_surface(top_, dim_.interior.w, top_->h));
+	// Too much typing is bad for you.
+	const SDL_Rect& i = dim_.interior;
+	const SDL_Rect& e = dim_.exterior;
 
-	if(top_image != nullptr) {
-		video_.blit_surface(dim_.interior.x, dim_.exterior.y, top_image);
+	if(top_) {
+		draw::blit(top_, {i.x, e.y, i.w, top_.h()});
 	}
 
-	surface bot_image(scale_surface(bot_, dim_.interior.w, bot_->h));
-
-	if(bot_image != nullptr) {
-		video_.blit_surface(dim_.interior.x, dim_.interior.y + dim_.interior.h, bot_image);
+	if(bot_) {
+		draw::blit(bot_, {i.x, i.y + i.h, i.w, bot_.h()});
 	}
 
-	surface left_image(scale_surface(left_, left_->w, dim_.interior.h));
-
-	if(left_image != nullptr) {
-		video_.blit_surface(dim_.exterior.x, dim_.interior.y, left_image);
+	if(left_) {
+		draw::blit(left_, {e.x, i.y, left_.w(), i.h});
 	}
 
-	surface right_image(scale_surface(right_, right_->w, dim_.interior.h));
-
-	if(right_image != nullptr) {
-		video_.blit_surface(dim_.interior.x + dim_.interior.w, dim_.interior.y, right_image);
+	if(right_) {
+		draw::blit(right_, {i.x + i.w, i.y, right_.w(), i.h});
 	}
 
-	if(top_left_ == nullptr || bot_left_ == nullptr || top_right_ == nullptr || bot_right_ == nullptr) {
+	if(!top_left_ || !bot_left_ || !top_right_ || !bot_right_) {
 		return;
 	}
 
-	video_.blit_surface(dim_.interior.x - left_->w, dim_.interior.y - top_->h, top_left_);
-	video_.blit_surface(dim_.interior.x - left_->w, dim_.interior.y + dim_.interior.h + bot_->h - bot_left_->h, bot_left_);
-	video_.blit_surface(dim_.interior.x + dim_.interior.w + right_->w - top_right_->w, dim_.interior.y - top_->h, top_right_);
-	video_.blit_surface(dim_.interior.x + dim_.interior.w + right_->w - bot_right_->w, dim_.interior.y + dim_.interior.h + bot_->h - bot_right_->h, bot_right_);
-}
+	draw::blit(top_left_,
+		{i.x - left_.w(), i.y - top_.h(), top_left_.w(), top_left_.h()});
 
-void dialog_frame::clear_background()
-{
-	delete restorer_;
-	restorer_ = nullptr;
+	draw::blit(bot_left_, {
+		i.x - left_.w(),
+		i.y + i.h + bot_.h() - bot_left_.h(),
+		bot_left_.w(),
+		bot_left_.h()
+	});
+
+	draw::blit(top_right_, {
+		i.x + i.w + right_.w() - top_right_.w(),
+		i.y - top_.h(),
+		top_right_.w(),
+		top_right_.h(),
+	});
+
+	draw::blit(bot_right_, {
+		i.x + i.w + right_.w() - bot_right_.w(),
+		i.y + i.h + bot_.h() - bot_right_.h(),
+		bot_right_.w(),
+		bot_right_.h()
+	});
 }
 
 void dialog_frame::draw_background()
 {
-	if(auto_restore_) {
-		clear_background();
-		restorer_ = new surface_restorer(&video_, dim_.exterior);
-	}
-
 	if (dialog_style_.blur_radius) {
-		surface surf = ::get_surface_portion(video_.getSurface(), dim_.exterior);
-		surf = blur_surface(surf, dialog_style_.blur_radius);
-		sdl_blit(surf, nullptr, video_.getSurface(), &dim_.exterior);
+		// This is no longer used by anything.
+		// The only thing that uses dialog_frame is help/help.cpp,
+		// and it uses the default style with no blur.
+		ERR_DP << "GUI1 dialog_frame blur has been removed";
 	}
 
-	if(bg_ == nullptr) {
-		ERR_DP << "could not find dialog background '" << dialog_style_.panel << "'" << std::endl;
+	if (!bg_) {
+		ERR_DP << "could not find dialog background '" << dialog_style_.panel << "'";
 		return;
 	}
-	for(int i = 0; i < dim_.interior.w; i += bg_->w) {
-		for(int j = 0; j < dim_.interior.h; j += bg_->h) {
+
+	auto clipper = draw::reduce_clip(dim_.interior);
+	for(int i = 0; i < dim_.interior.w; i += bg_.w()) {
+		for(int j = 0; j < dim_.interior.h; j += bg_.h()) {
 			SDL_Rect src {0,0,0,0};
-			src.w = std::min(dim_.interior.w - i, bg_->w);
-			src.h = std::min(dim_.interior.h - j, bg_->h);
+			src.w = std::min(dim_.interior.w - i, bg_.w());
+			src.h = std::min(dim_.interior.h - j, bg_.h());
 			SDL_Rect dst = src;
 			dst.x = dim_.interior.x + i;
 			dst.y = dim_.interior.y + j;
-			sdl_blit(bg_, &src, video_.getSurface(), &dst);
+			draw::blit(bg_, dst);
 		}
 	}
 }
 
-SDL_Rect dialog_frame::draw_title(CVideo* video)
+rect dialog_frame::draw_title(bool actually_draw)
 {
-	SDL_Rect rect = CVideo::get_singleton().screen_area();
-	return font::pango_draw_text(video, rect, font::SIZE_TITLE, font::TITLE_COLOR,
-	                       title_, dim_.title.x, dim_.title.y, false, font::pango_text::STYLE_NORMAL);
+	rect r = video::game_canvas();
+	return font::pango_draw_text(
+		actually_draw, r, font::SIZE_TITLE, font::TITLE_COLOR, title_,
+		dim_.title.x, dim_.title.y, false, font::pango_text::STYLE_NORMAL
+	);
 }
 
 void dialog_frame::draw()
 {
-	if (!dirty_)
-		return;
-
 	//draw background
 	draw_background();
 
@@ -339,10 +333,17 @@ void dialog_frame::draw()
 
 	//draw title
 	if (!title_.empty()) {
-		draw_title(&video_);
+		draw_title(true);
+	}
+}
+
+void dialog_frame::layout()
+{
+	if (!dirty_) {
+		return;
 	}
 
-	//draw buttons
+	// Layout buttons
 	SDL_Rect buttons_area = dim_.button_row;
 	if(buttons_ != nullptr) {
 #ifdef OK_BUTTON_ON_RIGHT
@@ -354,6 +355,7 @@ void dialog_frame::draw()
 		}
 	}
 
+	// Layout help button, if any
 	if(help_button_ != nullptr) {
 		help_button_->set_location(dim_.interior.x+ButtonHPadding, buttons_area.y);
 	}
@@ -361,4 +363,17 @@ void dialog_frame::draw()
 	dirty_ = false;
 }
 
+bool dialog_frame::expose(const rect& region)
+{
+	DBG_DP << "dialog_frame::expose " << region;
+	// Just draw everthing.
+	draw();
+	return true;
+}
+
+rect dialog_frame::screen_location()
+{
+	return dim_.exterior;
+}
+
 }
