@@ -47,6 +47,27 @@ static lg::log_domain log_unit("unit");
 static lg::log_domain log_wml("wml");
 #define ERR_WML LOG_STREAM(err, log_wml)
 
+namespace {
+/**
+ * Value of attack_type::num_recursion_ at which allocations of further recursion_guards fail. This
+ * value is used per weapon, so if two weapon specials are depending on each other being active then
+ * with ATTACK_RECURSION_LIMIT = 3 the recursion could go 6 levels deep (and then return false on
+ * the 7th call to matches_simple_filter).
+ *
+ * The counter is checked at the start of matches_simple_filter, and even the first level needs an
+ * allocation; setting the limit to zero would make matches_simple_filter always return false.
+ *
+ * With the recursion limit set to 1, the following tests fail; they just need a reasonable depth.
+ * event_test_filter_attack_specials
+ * event_test_filter_attack_opponent_weapon_condition
+ * event_test_filter_attack_student_weapon_condition
+ *
+ * With the limit set to 2, all tests pass, but as the limit only affects cases that would otherwise
+ * lead to a crash, it seems reasonable to leave a little headroom for more complex logic.
+ */
+constexpr unsigned int ATTACK_RECURSION_LIMIT = 4;
+};
+
 attack_type::attack_type(const config& cfg) :
 	self_loc_(),
 	other_loc_(),
@@ -97,12 +118,50 @@ std::string attack_type::accuracy_parry_description() const
 	return s.str();
 }
 
+namespace {
+/**
+ * Print "Recursion limit reached" log messages, including deduplication if the same problem has
+ * already been logged.
+ */
+void show_recursion_warning(const attack_type& attack, const config& filter) {
+	// This function is only called when the recursion limit has already been reached, meaning the
+	// filter has already been parsed multiple times, so I'm not trying to optimize the performance
+	// of this; it's merely to prevent the logs getting spammed. For example, each of
+	// four_cycle_recursion_branching and event_test_filter_attack_student_weapon_condition only log
+	// 3 unique messages, but without deduplication they'd log 1280 and 392 respectively.
+	static std::vector<std::tuple<std::string, std::string>> already_shown;
+
+	auto identifier = std::tuple<std::string, std::string>{attack.id(), filter.debug()};
+	if(utils::contains(already_shown, identifier)) {
+		return;
+	}
+
+	std::string_view filter_text_view = std::get<1>(identifier);
+	utils::trim(filter_text_view);
+	ERR_UT << "Recursion limit reached for weapon '" << attack.id()
+		<< "' while checking filter '" << filter_text_view << "'";
+
+	// Arbitrary limit, just ensuring that having a huge number of specials causing recursion
+	// warnings can't lead to unbounded memory consumption here.
+	if(already_shown.size() > 100) {
+		already_shown.clear();
+	}
+	already_shown.push_back(std::move(identifier));
+}
+
 /**
  * Returns whether or not *this matches the given @a filter, ignoring the
  * complexities introduced by [and], [or], and [not].
  */
-static bool matches_simple_filter(const attack_type & attack, const config & filter, const std::string& tag_name)
+bool matches_simple_filter(const attack_type& attack, const config& filter, const std::string& check_if_recursion)
 {
+	//update and check variable_recursion for prevent check special_id/type_active in case of infinite recursion.
+	attack_type::recursion_guard filter_lock= attack.update_variables_recursion();
+	if(!filter_lock) {
+		show_recursion_warning(attack, filter);
+		return false;
+	}
+
 	const std::set<std::string> filter_range = utils::split_set(filter["range"].str());
 	const std::string& filter_damage = filter["damage"];
 	const std::string& filter_attacks = filter["number"];
@@ -145,8 +204,11 @@ static bool matches_simple_filter(const attack_type & attack, const config & fil
 		return false;
 
 	if (!filter_type.empty()){
-		//if special is type "damage_type" then check attack.type() only for don't have infinite recursion by calling damage_type() below.
-		if(tag_name == "damage_type"){
+		// Although there's a general guard against infinite recursion, the "damage_type" special
+		// should always use the base type of the weapon. Otherwise it will flip-flop between the
+		// special being active or inactive based on whether ATTACK_RECURSION_LIMIT is even or odd;
+		// without this it will also behave differently when calculating resistance_against.
+		if(check_if_recursion == "damage_type"){
 			if (filter_type.count(attack.type()) == 0){
 				return false;
 			}
@@ -253,29 +315,30 @@ static bool matches_simple_filter(const attack_type & attack, const config & fil
 	// Passed all tests.
 	return true;
 }
+} // anonymous namespace
 
 /**
  * Returns whether or not *this matches the given @a filter.
  */
-bool attack_type::matches_filter(const config& filter, const std::string& tag_name) const
+bool attack_type::matches_filter(const config& filter, const std::string& check_if_recursion) const
 {
 	// Handle the basic filter.
-	bool matches = matches_simple_filter(*this, filter, tag_name);
+	bool matches = matches_simple_filter(*this, filter, check_if_recursion);
 
 	// Handle [and], [or], and [not] with in-order precedence
 	for (const config::any_child condition : filter.all_children_range() )
 	{
 		// Handle [and]
 		if ( condition.key == "and" )
-			matches = matches && matches_filter(condition.cfg, tag_name);
+			matches = matches && matches_filter(condition.cfg, check_if_recursion);
 
 		// Handle [or]
 		else if ( condition.key == "or" )
-			matches = matches || matches_filter(condition.cfg, tag_name);
+			matches = matches || matches_filter(condition.cfg, check_if_recursion);
 
 		// Handle [not]
 		else if ( condition.key == "not" )
-			matches = matches && !matches_filter(condition.cfg, tag_name);
+			matches = matches && !matches_filter(condition.cfg, check_if_recursion);
 	}
 
 	return matches;
@@ -573,6 +636,50 @@ bool attack_type::describe_modification(const config& cfg,std::string* descripti
 	return true;
 }
 
+attack_type::recursion_guard attack_type::update_variables_recursion() const
+{
+	if(num_recursion_ < ATTACK_RECURSION_LIMIT) {
+		return recursion_guard(*this);
+	}
+	return recursion_guard();
+}
+
+attack_type::recursion_guard::recursion_guard() = default;
+
+attack_type::recursion_guard::recursion_guard(const attack_type& weapon)
+	: parent(weapon.shared_from_this())
+{
+	weapon.num_recursion_++;
+}
+
+attack_type::recursion_guard::recursion_guard(attack_type::recursion_guard&& other)
+{
+	std::swap(parent, other.parent);
+}
+
+attack_type::recursion_guard::operator bool() const {
+	return bool(parent);
+}
+
+attack_type::recursion_guard& attack_type::recursion_guard::operator=(attack_type::recursion_guard&& other)
+{
+	// This is only intended to move ownership to a longer-living variable. Assigning to an instance that
+	// already has a parent implies that the caller is going to recurse and needs a recursion allocation,
+	// but is accidentally dropping one of the allocations that it already has; hence the asserts.
+	assert(this != &other);
+	assert(!parent);
+	std::swap(parent, other.parent);
+	return *this;
+}
+
+attack_type::recursion_guard::~recursion_guard()
+{
+	if(parent) {
+		assert(parent->num_recursion_ > 0);
+		parent->num_recursion_--;
+	}
+}
+
 void attack_type::write(config& cfg) const
 {
 	cfg["description"] = description_;
